package market;

import java.util.Iterator;

import data.Catalog;
import data.CatalogItem;
import data.DataBasket;
import data.events.VetoException;
import data.ooimpl.CatalogImpl;
import data.ooimpl.CountingStockImpl;

/**
 * CountingStockImpl that always contains the same keys as its {@linkplain CArticleCatalog} source catalog,
 * even if no StockItems have been added or some have been removed completely.
 * Normally the keys of StockItems are removed when their number drops to zero. But this prevents
 * StockItems with an amount of zero to be displayed in tables.
 */
public class CSOffer extends CountingStockImpl {

    private Integer iNull = new Integer(0);

    /**
     * @param sName the Stock's name.
     * @param cat the source catalog.
     */
    public CSOffer(String sName, CatalogImpl cat) {
        super(sName, cat);
        addZeros(null);
    }

    /**
     * Removes StockItems from the Stock. If the item has been removed completely, its
     * index is re-added and its amount is set to 0.
     *
     * @param sKey the ID of the StockItem to be removed.
     * @param nCount the number of Items to be removed.
     * @param db the belonging DataBasket.
     */
    public void remove(String sKey, int nCount, DataBasket db){
        try {
            super.remove(sKey, nCount, db);
        }
        catch (VetoException ve) {
            System.err.println(ve.getMessage());
        }
        if (!contains(sKey, db)) {
            getItemsContainer().put(sKey, iNull);
        }
    }

    /**
     * Checks, which CatalogItems have no appropriate StockItems.
     * If one is found the key of the item is added to the stock.
     * The number of StockItems is set to 0.
     *
     * @param db the DataBasket to be used.
     */
    public void addZeros(DataBasket db) {
        Iterator it = m_ciCatalog.iterator(null, false);
        while (it.hasNext()) {
            String sKey = ((CatalogItem)it.next()).getName();
            if (!contains(sKey, db)) {
                getItemsContainer().put(sKey, iNull);
            }
        }
    }


    /**
     * Changes the StockItems of this stock according to a new Catalog<br><br>
     * All elements that appear in both the old and the new Catalog will be kept.<br>
     * All elements that do not appear in the new Catalog will be deleted.<br>
     * All elements that have not appeared in the old Catalog will be added
     * and their number is set to 0.<br>
     * This method is used by the filters that filter Stocks by their item's category.
     *
     * @param cat the new Catalog to be set.
     */
    public void changeArticleCatalog(Catalog cat) throws VetoException {
        Iterator it = null;
        //convert the given catalog to a CatalogImpl and make it the new base
        Catalog oldBase = m_ciCatalog;
        if (cat instanceof CatalogImpl) {   //Catalog instance of CatalogImpl => leave it
            m_ciCatalog = (CatalogImpl)cat;
        } else {                            //Catalog not instance of CatalogImpl (e.g. Filter)
            CatalogImpl c = new CatalogImpl(m_ciCatalog.getName());
            it = cat.keySet(null).iterator();
            while (it.hasNext()) {                      //copy all CatalogItems from cat to new
                String nextKey = (String)it.next();     //created CatalogImpl
                CatalogItem ci = oldBase.get(nextKey, null, false);
                c.add(ci, null);
            }
            m_ciCatalog = c;                            //make new CatalogImpl the new base
        }
        //iterate through new base and remove all unneeded StockItems from this stock
        it = this.keySet(null).iterator();
        int nrOfItems = 0;
        while (it.hasNext()) {
            String sKey = (String)it.next();
            nrOfItems = countItems(sKey, null);
            if (!contains(sKey, null)) {
                if (nrOfItems > 0) {
                    try {
                        super.remove(sKey, nrOfItems, null);
                    }
                    catch (VetoException ve) {
                        ve.printStackTrace();
                        throw ve;
                    }
                } else {
                    getItemsContainer().remove(sKey);
                }
            }
            addZeros(null);
        }
    }

    /**
     * @return a clone of CSOffer with its StockItems.
     */
    public Object clone() {
        CSOffer cso = new CSOffer(getName(), m_ciCatalog);
        Iterator it = this.keySet(null).iterator();
        while (it.hasNext()) {
            String sKey = (String)it.next();
            int amount = countItems(sKey, null);
            if (amount > 0) {                       //add all items with one or more elements
                cso.add(sKey, amount, null);        //(adding 0 elements is not allowed by the Framework)
            }
            addZeros(null);
        }
        return cso;
    }
}