package market;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.StringTokenizer;

import data.NumberValue;
import data.Value;
import data.ooimpl.CatalogItemImpl;
import data.ooimpl.StockItemImpl;
import data.swing.CountingStockTableModel;

/**
 * A collection of methods to convert Strings, numbers and other Objects into another format.
 */
public abstract class Conversions {

    /**
     * Converts a double into a String with a fixed number of decimal places.
     * @param d the double to be converted.
     * @param decimalPlace the number of decimal places
     * @return the formatted String.
     */
    public static String fixedDecimal(double d, int decimalPlace) {
        char[] dp = new char[2+decimalPlace];
        for (int i=0; i<2+decimalPlace; i++) dp[i] = '0';
        dp[1] = '.';
        String pattern = String.valueOf(dp);
        DecimalFormat df = new DecimalFormat(pattern);
        return df.format(d);
    }

    /**
     * Converts a Value that represents an amount of money in cents into a String,
     * that represents the money in Euros (or Dollars...).
     * @param v the value to be converted.
     * @return the formatted String.
     */
    public static String valueToCurrency(Value v) {
        return fixedDecimal(new Double(v.toString()).doubleValue()/100, 2);
    }

    /**
     * Converts a Value that represents an amount of money in cents into a String,
     * that represents the money in Euros (or Dollars...).
     * @param v the value to be converted.
     * @param extra an extra String added behind the converted value.
     * @return the formatted String.
     */
    public static String valueToCurrency(Value v, String extra) {
        return valueToCurrency(v) + extra;
    }

    /**
     * Converts a double that represents an amount of money in cents into a String.
     * that represents the money in Euros (or Dollars...).
     * @param d the double to be converted.
     * @return the formatted String.
     */
     public static String doubleToCurrency(double d) {
        return fixedDecimal(d/100, 2);
    }

    /**
     * Converts a double that represents an amount of money in cents into a String,
     * that represents the money in Euros (or Dollars...).
     * @param d the double to be converted.
     * @param extra an extra String added behind the converted double.
     * @return the formatted String.
     */
     public static String doubleToCurrency(double d, String extra) {
        return doubleToCurrency(d) + extra;
    }

    /**
     * Converts a Value to a Double.
     * @param v the Value to be converted.
     * @return the converted Value as Double.
     */
    public static Double valueToDouble(Value v) {
        return new Double(v.toString());
    }

    /**
     * Converts a currency string into an int.
     *
     * @param s the string to be converted.
     * @param euroToCents if <code>true</code>, the string is interpreted as Euro value and multiplied by
     * 100, otherwise the string is assumed to represent cents already.
     * @return the money as cents.
     */
    public static int currencyToInt(String s, boolean euroToCents) {
        int multiply = euroToCents ? 100 : 1;
        return new Double(Double.valueOf(convertComma(s)).doubleValue()*multiply).intValue();
    }

    /**
     * Cuts a double after the specified number of decimal places.
     * @param d the double to be rounded.
     * @param decimalPlace the decimal place after which is cut.
     * @return the rounded double.
     */
    public static double round(double d, int decimalPlace) {
        BigDecimal myDec = new BigDecimal(d);
        return myDec.setScale(decimalPlace, BigDecimal.ROUND_HALF_UP).doubleValue();
    }

    /**
     * Converts all commas in a string into points (e.g. 1,225 => 1.225).
     * @param s the String to be converted.
     * @return the converted String.
     */
    public static String convertComma(String s) {
        String newString = "";
        if (s.startsWith(",")) newString = ".";
        StringTokenizer st = new StringTokenizer(s, ",");
        while (st.hasMoreTokens()) {
            newString += st.nextToken();
            if (st.hasMoreTokens()) newString += ".";
        }
        try {
            return new Double(Double.parseDouble(newString)).toString();
        }
        catch (Exception e) {
            return null;
        }
    }

    /**
     * Casts a CIArticle, a CountingStockTableModel.Record or a StockItemImpl into a CIArticle
     * if possible. If the object cannot be converted <code>null</code> is returned.
     * @param record the Object to be cast.
     * @return the cast object.
     */
    public static CIArticle recordToCIArticle(Object record) {
        if (record instanceof CIArticle) return (CIArticle)record;
        if (record instanceof CountingStockTableModel.Record)
                return (CIArticle)((CountingStockTableModel.Record)record).getDescriptor();
        if (record instanceof StockItemImpl)
                return (CIArticle)((StockItemImpl)record).getAssociatedItem(null);
        return null;
    }

    /**
     * Creates a CatalogItemImpl from a UCustomer.
     * @param customer the customer to create the CatalogItemImpl from.
     * @return the created CatalogItemImpl.
     */
    public static CatalogItemImpl customerToCI(UCustomer customer){
        CatalogItemImpl cii = new CatalogItemImpl(customer.getName()){
            protected CatalogItemImpl getShallowClone() {
                return null;
            }
        };
        return cii;
    }

    /**
     * Converts a date string ("dd.MM.yyyy") to a Calendar date.
     * @param dateString the String representing the date.
     * @return the Calendar.
     */
    public static Calendar stringToCalendar(String dateString) {
        boolean error = false;
        SimpleDateFormat sdf = new SimpleDateFormat ("dd.MM.yyyy");
        sdf.setLenient(false);
        try {
            sdf.parse(dateString);
            //Parsing also accepts dates without left handed zeros,
            //so this extra check for the String's length is being done
            error = (dateString.length() != 10);
        }
        catch (java.text.ParseException e)  {
            error = true;
        }
        if (error) return null;
        else {
            StringTokenizer strTok = new StringTokenizer((String)dateString,".",false);
            int day = new Integer(strTok.nextToken()).intValue();
            int month = new Integer(strTok.nextToken()).intValue();
            int year = new Integer(strTok.nextToken()).intValue();
            return new MarketCalendar(year, month - 1, day);
        }
    }


    /**
     * @return today's (today means "realtime today") date with time set to 0:00.00.
     */
    public static Calendar createToday() {
        Calendar c = new MarketCalendar();
        return new MarketCalendar(c.get(Calendar.YEAR), c.get(Calendar.MONTH),
                c.get(Calendar.DATE));
    }

    /**
     * Converts a Value to an int.
     * @param v the Value to be converted.
     * @return the converted Value as int.
     */
    public static int valueToInt(Value v){
        if(v instanceof NumberValue) return ((NumberValue)v).getValue().intValue();
        return 0;
    }

    /**
     * Computes the difference between two dates in days.
     * @param cEarly the first date.
     * @param cLate the second date.
     * @return the difference in days.
     */
    public static int dayDifference(Calendar cEarly, Calendar cLate) {
        long l1 = cEarly.getTime().getTime();
        long l2 = cLate.getTime().getTime();
        return new Long((l2 - l1)/86400000).intValue();
    }
}