package market;

import java.util.Iterator;
import java.util.TreeMap;

import market.event.OfferEventListener;
import users.UserManager;
import data.CountingStock;
import data.StockItem;
import data.events.VetoException;
import data.ooimpl.CountingStockImpl;
import data.ooimpl.StockItemImpl;

/**
 * A StockItemImpl that is used as a representation of a {@link UCustomer}.
 * It can be used to add it to an instance of {@link SSListenable} which represents
 * a waiting-queue at the tills or at the warehouse.
 * The implementation of {@link OfferEventListener} makes it possible to inform about the whole
 * inventory of the market this is useful if shortages occurred,
 * the only correction of the {@link CSOffer} wouldn't be suffice because already selled articles and
 * those in the shopping-basket of customers aren`t in it anymore.
 */
public class SICustomer extends StockItemImpl implements OfferEventListener{

    /**
     * A short message of what articles can`t get sold.
     * Only important at the till-queue.
     */
    private String missingArticles = new String("");

    /**
     * The time since this SICustomer exist
     * used to prove when this SICustomer arrived in the Stock.
     * Only important at the till-queue.
     */
    private Long l_tillTime;

    /**
     * A TreeMap containing all orders which aren`t yet processed.
     */
    private TreeMap tm_orders = new TreeMap();

    /**
    * The monitor used to synchronize access to the TreeMap of CSOrders.
    */
    private transient Object o_ordersLock;

    /**
     * Stores to which Stock this SICustomer belongs to.
     */
    private String queue;


    //################################### Constructor ###########################################

    /**
     * @param customer name of the {@link UCustomer} as the new ID, all keys of the UCustomers
     * are stored in a CatalogImpl.
     */
    public SICustomer(String customer){
        super(customer);
        l_tillTime = new Long(System.currentTimeMillis());
    }

    /**
     * Adds a new {@link CSOrder} to the order-set.
     *
     * @param cs the StockItems of this CountingStock will be added to the new CSOrder.
     * @param active if true the new CSOrder will be active, otherwise not.
     */
    public void addOrderToQueue(CountingStock cs, boolean active){
        CSOrder cso = CSOrder.create(this.getName(), active);
        cso.addStock(cs, null, true);
        synchronized(getOrderLock()){
            tm_orders.put(cso.getTime(),cso);
        }
        SMarket.fireUpdateWorkerScreen();
    }

    /**
     * @return the oldest, active {@link CSOrder} of this SICustomer and removes it from the order-set.
     */
    public CSOrder removeOrderFromQueue(){
        CSOrder cs = null;
        synchronized(getOrderLock()){
            Iterator it = tm_orders.values().iterator();
            while(it.hasNext()){
                cs = (CSOrder)it.next();
                if(cs.isActive()){
                    tm_orders.remove(cs.getTime());
                    break;
                }
            }
        }
        SMarket.fireUpdateWorkerScreen();
        return cs;
    }

    /**
     * Puts a given {@link CSOrder} back to the order-set of this SICustomer.
     *
     * @param order the CSOrder that will be put back.
     */
    public void rollbackOrder(CSOrder order){
        synchronized(getOrderLock()){
            tm_orders.put(order.getTime(), order);
        }
        SMarket.fireUpdateWorkerScreen();
    }

    /**
     * @return an identical SICustomer.
     */
    public Object clone() {
        SICustomer sic = new SICustomer(this.getName());
        sic.l_tillTime = this.l_tillTime;
        sic.tm_orders = this.tm_orders;
        return sic;
    }

    /**
    * @return the monitor used to synchronize access to the TreeMap of CSOrders.
    */
    private final Object getOrderLock() {
        if (o_ordersLock == null) {
            o_ordersLock = new Object();
        }
        return o_ordersLock;
    }

    /**
     * @return the number of {@link CSOrder}s of this SICustomer.
     *
     * @param active if true only active CSOrders will be counted.
     */
    public int getOrderCount(boolean active){
        int i = 0;
        synchronized(getOrderLock()){
            Iterator it = tm_orders.values().iterator();
            while(it.hasNext()){
                CSOrder cso = (CSOrder)it.next();
                if(active){
                    if(cso.isActive()) i++;
                }
                else i++;
            }
        }
        return i;
    }

    /**
     * @return the {@link UCustomer} this SICustomer represents.
     */
    public UCustomer getCustomer(){
        return (UCustomer)UserManager.getGlobalUM().getUser(this.getName());
    }

    public String getMissingArticles() {
        return missingArticles;
    }

    /**
     * @return the waiting-time of the oldest order of this SICustomer in milliseconds.
     */
    public Long getOrderQueueTime(){
        Long l = new Long(System.currentTimeMillis());
        synchronized(getOrderLock()){
            Iterator it = tm_orders.values().iterator();
            while(it.hasNext()){
                CSOrder cso = (CSOrder)it.next();
                if(cso.isActive()){
                    l = cso.getTime();
                    break;
                }
            }
        }
        return l;
    }

    /**
     * Compares this SICustomer to another one using l_tillTime.
     *
     * @param o a SICustomer that should be compared to this one
     * @return 0 if the argument SICustomer is equal to this one;
     * a value less than 0 if this SICustomer is older than the given one;
     * and else a value greater than 0.
     */
    public int compareTo(Object o) {
        return l_tillTime.compareTo(((SICustomer)o).l_tillTime);
    }



    /**
     * Reaction on event: An article is unavaible.
     *
     * @param articleKey the key of the unavaible article.
     */
    public void offerEmpty(String articleKey) {
        // if this SICustomer contains to the till-queue
        if(queue.compareTo(SMarket.STK_TILLQUEUE)==0){
            CountingStockImpl csi = getCustomer().getShoppingBasket();
            if(csi.contains(articleKey, null)){
                missingArticles += new String("\n"+String.valueOf(csi.countItems(articleKey, null))  +
                    "x " + SMarket.getArticleCatalog().get(articleKey).getArticleName());
                SMarket.getOffer().add(articleKey, csi.countItems(articleKey, null), null);
                try {
                    csi.remove(articleKey, csi.countItems(articleKey, null), null);
                } catch (VetoException e) {
                    System.err.println(e.getMessage());
                }
            }
        }
        // if this SICustomer contains to the warehouse-queue
        if(queue.compareTo(SMarket.STK_WAREHOUSEQUEUE)==0){
            CountingStockImpl csi_current = new CountingStockImpl("current", SMarket.getArticleCatalog());
            synchronized(getOrderLock()){
                Iterator it = tm_orders.values().iterator();
                while(it.hasNext()){
                    CSOrder cs = (CSOrder)it.next();
                    if(cs.contains(articleKey, null)){
                        csi_current.add(articleKey, cs.removeAll(articleKey), null);
                    }
                }
            }
            if(csi_current.contains(articleKey, null)) addOrderToQueue(csi_current, false);
        }
    }

    /**
     * Reaction on event: a new delivery arrived the market.
     */
    public void wakeUpOrders() {
        synchronized(getOrderLock()){
            Iterator it = tm_orders.values().iterator();
            while(it.hasNext()){
                CSOrder cso = (CSOrder)it.next();
                if(!cso.isActive() && SMarket.getOffer().containsStock(cso, null)){
                    Iterator it_article = cso.iterator(null, false);
                    while(it_article.hasNext()){
                        String key = ((StockItem)it_article.next()).getName();
                        try {
                            SMarket.getOffer().remove(key, null);
                        } catch (VetoException e) {
                            System.err.println(e.getMessage());
                        }
                    }
                    cso.setActive(true);
                }
            }
        }
    }

    /**
     * Reaction on event: a SProcessWorker needs the count of all existing articles.
     *
     * @param articleKey the name of the article.
     * @param spw the SProcessWorker that is affected.
     */
    public void countArticles(String articleKey, SProcessWorker spw) {
        int count = 0;
        if(queue.compareTo(SMarket.STK_TILLQUEUE)==0){
            if(getCustomer().getShoppingBasket().contains(articleKey, null)){
                count += getCustomer().getShoppingBasket().countItems(articleKey, null);
                spw.addDatabaseCount(count, SProcessWorker.TILLQUEUE);
            }
        }
        if(queue.compareTo(SMarket.STK_WAREHOUSEQUEUE)==0){
            synchronized(getOrderLock()){
                Iterator it = tm_orders.values().iterator();
                while(it.hasNext()){
                    count += ((CountingStock)it.next()).countItems(articleKey, null);
                }
                if(count>0){
                    spw.addDatabaseCount(count, SProcessWorker.WAREHOUSEQUEUE);
                }
            }
        }
    }

    /**
     * Adds a UCustomer to the global till-queue(a Stock).
     *
     * @param customer the UCustomer that will be added.
     */
    public static void addToTillQueue(UCustomer customer){
        SICustomer sic = new SICustomer(customer.getName());
        SMarket.getTillQueue().add(sic, null);
        sic.queue = SMarket.STK_TILLQUEUE;
    }

    /**
     * Adds the shoppingbasket of a UCustomer as a CSOrder to the warehouse-queue(a Stock).
     *
     * @param customer the UCustomer who`s shoppingbasket will be added.
     */
    public static void addToOrderQueue(UCustomer customer){
        SSListenable queue = SMarket.getWarehouseQueue();
        String key = customer.getName();
        Iterator it = queue.get(key, null, true);
        if(it.hasNext()){
            SICustomer sic = (SICustomer)it.next();
            sic.addOrderToQueue(sic.getCustomer().getShoppingBasket(), true);
        }
        else{
            SICustomer sic = new SICustomer(key);
            queue.add(sic, null);
            sic.addOrderToQueue(sic.getCustomer().getShoppingBasket(), true);
            sic.queue = SMarket.STK_WAREHOUSEQUEUE;
        }
    }
}
