package market;

import java.awt.Rectangle;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import javax.swing.JFrame;

import log.Log;
import market.event.MarketEventListener;
import market.resource.IconSetter;
import market.statistics.CCompleteStats;
import market.statistics.CCustomerStats;
import market.statistics.CISalesStats;
import market.statistics.CSalesStats;
import market.statistics.Statistics;
import sale.Action;
import sale.MenuSheet;
import sale.MenuSheetItem;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import sale.StepTimer;
import sale.Time;
import data.Catalog;
import data.CountingStock;
import data.IntegerValue;
import data.Value;
import data.ooimpl.CatalogImpl;
import data.ooimpl.StoringStockImpl;

/**
 * The Shop of this sale-application.
 */
public class SMarket extends Shop {

    public static final String CAT_ARTICLECATALOG = "Produktkatalog";
    public static final String CAT_OPENPURCHASE = "Erwartete Lieferungen";
    public static final String CAT_CUSTOMER = "Kunden";
    public static final String CAT_COMPLETE_SALESSTATS = "Komplette Statistik";
    public static final String STK_OFFER = "Angebot";
    public static final String STK_TILLQUEUE = "Kunden-Warteschlange";
    public static final String STK_WAREHOUSEQUEUE = "Auftrags-Warteschlange";
    public static final String MARKET_CLOSES_SHORT = "ACHTUNG: Der Markt schliet jetzt.";
    public static final String MARKET_CLOSES_LONG = "Der Markt schliet gleich. Beehren sie uns doch " +
            "morgen wieder.";
    public static final String MARKET_CLOSES_NOT = "Der Markt hat doch noch eine Weile geffnet.";
    public static final String MARKET_CLOSED = "Der Markt hat geschlossen.";
    public static final String MARKET_OPENED = "Der Markt hat geffnet.";
    private static final String SHOP_CAPTION = "Gromarkt Sohn & Sohn";

    private MenuSheetItem msiNewCustomer;
    private MenuSheetItem msiCustomer;
    private MenuSheetItem msiSeller;
    private MenuSheetItem msiWorker;
    private MenuSheetItem msiManager;
    private Calendar dateOfOpening;


    /**
     * The {@link Options options}, which affect the computation of the discount and the
     * dismissal compensation.
     */
    private Options options = new Options();

    /**
     * The listeners listening to market events.
     */
    private MarketEventListener[] evl = new MarketEventListener [0];

    /**
     * Indicates if market ist open, soon to close, or closed.
     */
    private int open;

    /**
     * Indicates if the market's time has advanced.
     * This variable is set to true when the day changes. As soon as the market opens,
     * timeAdvanced will be false.<br>
     * That makes it possible to distinguish between the two possibilities
     * <ol><li>time has advanced and market can be opened</li>
     * <li>market has just closed and day has not been changed yet</li></ol>
     */
    private boolean timeAdvanced;

    /**
     * Catalog which contains all {@link CIArticle CIArticles}
     */
    private CatalogImpl c_articleCatalog;

    /**
     * Stock which contains all {@link CIArticle CIArticles} currently available
     */
    private CSOffer cs_offer;

    /**
     * Catalog which contains all orders placed by the manager that have not yet arrived.
     */
    private COpenPurchaseOrders c_openPurchaseOrders;

    /**
     * Catalog that stores all customers' names.
     * This catalog is needed for the customer stock ({@link #ss_tillQueue till queue}).
     */
    private CatalogImpl c_customer;

    /**
     * Catalog that stores sales statistics for the current day. This Catalog is used to make sure that
     * sales don't show up in the statistics unless the day-end closing has been completed.
     */
    private CSalesStats c_dailyStats;

    /**
     * Catalog that stores sales statistics (amount of sold articles, revenue from sold articles...)
     * for one month.
     */
    private CSalesStats c_monthlyStats;

    /**
     * Catalog that stores {@link CSalesStats sales statistics} of all passed months.
     */
    private CCompleteStats c_completeStats;

    /**
     * Catalog that stores the statistics of all market's customers. Those statistics are
     * time independend, that means, it is not split up into months.
     */
    private CCustomerStats c_customerStats;

    /**
     * The global queue of customers' orders waiting for warehouse-worker-processing.
     */
    private StoringStockImpl ss_warehouseQueue;

    /**
     * The global queue of customers waiting at the till.
     */
    private StoringStockImpl ss_tillQueue;

    /**
     * The money of the market.
     */
    private IntegerValue iv_account;


//####################### SalesPoint methods ################################################################

    /**
     * Creates an SMarket, initiates global Catalogs, Stocks and other settings.
     */
    public SMarket(Time time){
        super();
        setTheShop(this);
        setTimer(new StepTimer(time));
        setShopFrameBounds(new Rectangle(50,50,400,300));
        setShopFrameTitle(SHOP_CAPTION + "  -  " + getTime());
        IconSetter.setIcon(getShopFrame());
        open = 2; timeAdvanced = true; //shop starts in the morning
        c_articleCatalog = new CArticleCatalog(CAT_ARTICLECATALOG);
        addCatalog(c_articleCatalog);
        cs_offer = new CSOffer(STK_OFFER, c_articleCatalog);
        addStock(cs_offer);
        c_customer = new CatalogImpl(CAT_CUSTOMER);
        addCatalog(c_customer);
        ss_tillQueue = new SSListenable(STK_TILLQUEUE, c_customer);
        addStock(ss_tillQueue);
        ss_warehouseQueue = new SSListenable(STK_WAREHOUSEQUEUE, c_customer);
        addStock(ss_warehouseQueue);
        iv_account = new IntegerValue(0);
        //clone, otherwise dateOfOpening would change with current time
        dateOfOpening = (Calendar)getTime().clone();
    }

    /**
     * Returns the JFrame of the Shop window. The difference to the {@link Shop Shop's} getShopFrame()
     * method is, that this one is public, as the Shop's window is needed by a dialog.
     * @see JDDShowMessage#showMessageDialog(String, String)
     *
     * @return the Shop's frame.
     */
    public JFrame getShopFrame() {
        return super.getShopFrame();
    }

    /**
     * Starts the Shop and initializes statistics items.
     */
    public void start() {
        c_openPurchaseOrders = new COpenPurchaseOrders(CAT_OPENPURCHASE);
        addCatalog(c_openPurchaseOrders);
        c_monthlyStats = new CSalesStats(getYear(), getMonth());
        c_monthlyStats.initPriceHistory();
        addCatalog(c_monthlyStats);
        c_dailyStats = new CSalesStats(getYear(), getMonth());
        c_customerStats = new CCustomerStats();
        c_completeStats = new CCompleteStats(CAT_COMPLETE_SALESSTATS);
        addCatalog(c_completeStats);
        super.start();
        try {
            Log.setGlobalLogFile("marketlog.txt", true, false);
        }
        catch (IOException e) {
            System.err.println("Cannot set log file");
        }
        setOpen(0);
    }

    /**
     * Closes the SMarket.
     */
    public void quit() {
        if (shutdown (false)) {
            System.exit(0);
        }
    }

    /**
     * Sets the icon when market is loaded from persistence file.
     */
    public void resume() {
        super.resume();
        IconSetter.setIcon(getShopFrame());
    }

    /**
     * Creates the MenuSheet of the SMarket
     *
     * @return the MenuSheet of the SMarket
     */
    public MenuSheet createShopMenuSheet(){
        MenuSheet msMenuBar = new MenuSheet("Menubar");
        MenuSheet msShop = (MenuSheet)super.createShopMenuSheet().getTaggedItem(Shop.SHOP_MENU_TAG, false);
        MenuSheet msLogOn = new MenuSheet("Anmeldung");
        MenuSheet msSimulation = new MenuSheet("Simulation");
        msiNewCustomer = new MenuSheetItem("Kundenregistrierung", SProcessCustomerEditProfile.create());
        msiCustomer = new MenuSheetItem("Kunde", SProcessLogOn.createLogOnProcess(UMUserBase.CUSTOMER));
        msiSeller = new MenuSheetItem("Kasse", SProcessLogOn.createLogOnProcess(UMUserBase.SELLER));
        msiWorker = new MenuSheetItem("Lager", SProcessLogOn.createLogOnProcess(UMUserBase.WAREHOUSE_WORKER));
        msiManager = new MenuSheetItem("Manager", SProcessLogOn.createLogOnProcess(UMUserBase.MANAGER));
        MenuSheetItem msiAdvanceTime = new MenuSheetItem("Zu Wunschdatum vor", new Action() {
                public void doAction(SaleProcess p, SalesPoint sp) {
                    if (isOpen()) {
                        JDDShowMessage.showMessageDialog(
                                "Die Zeit kann erst weitergeschaltet werden,\nwenn der Markt " +
                                "geschlossen ist.\n\n (Manager -> System -> ffnen/Schlieen)", "Fehler");
                    } else {
                        SMarket.getTheShop().addSalesPoint(new SPTime());
                    }
        }});
        msLogOn.add(msiNewCustomer);
        msLogOn.add(msiCustomer);
        msLogOn.add(msiSeller);
        msLogOn.add(msiWorker);
        msLogOn.add(msiManager);
        msSimulation.add(msiAdvanceTime);
        msMenuBar.add(msShop);
        msMenuBar.add(msLogOn);
        msMenuBar.add(msSimulation);
        return msMenuBar;
    }

    /**
     * Adds a {@link SalesPoint} to the market attaches a {@link MarketEventListener} to it.
     *
     * @param sp the SalesPoint to be added.
     */
    public void addSalesPoint(SalesPoint sp) {
        super.addSalesPoint(sp);
        if (sp instanceof SPListenable) SMarket.addEventListener((SPListenable)sp);
        if (sp instanceof SPCustomer) SMarket.addEventListener((SPCustomer)sp);
    }

    /**
     * Removes a {@link SalesPoint} to the market detachses its {@link MarketEventListener}.
     *
     * @param sp the SalesPoint to be removed.
     */
    public void removeSalesPoint(SalesPoint sp) {
        super.removeSalesPoint(sp);
        if (sp instanceof SPListenable) SMarket.removeEventListener((SPListenable)sp);
        if (sp instanceof SPCustomer) SMarket.removeEventListener((SPCustomer)sp);
    }



//##################### Our methods #########################################################################

////////////////////////////////////////////////////////////////////////////////////////////////////////
// Account (getting and setting)
////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Adds a Value to the markets account.
     *
     * @param money the Value that will be added.
     */
    public static void addToAccount(Value money) {
        ((SMarket)Shop.getTheShop()).iv_account.addAccumulating(money);
    }

    /**
     * Subtracts a Value from the markets account.
     *
     * @param money the Value that will be substracted.
     */
    public static void subtractFromAccount(Value money) {
        ((SMarket)Shop.getTheShop()).iv_account.subtractAccumulating(money);
    }

    /**
     * @return the account of the market as Value.
     */
    public static Value getAccount(){
        return ((SMarket)Shop.getTheShop()).iv_account;
    }

    public static Options getOptions() {
        return getTheMarket().options;
    }


////////////////////////////////////////////////////////////////////////////////
// Shortcuts
////////////////////////////////////////////////////////////////////////////////

    /**
     * @return the singleton instance of SMarket.
     */
    public static SMarket getTheMarket() {
        return (SMarket)getTheShop();
    }

    /**
     * Shortcut for catalog with all articles.
     */
    public static CArticleCatalog getArticleCatalog() {
      return (CArticleCatalog)getTheShop().getCatalog(CAT_ARTICLECATALOG);
    }

    /**
     * Shortcut to the market's current stock.
     */
    public static CSOffer getOffer() {
      return (CSOffer)getTheShop().getStock(STK_OFFER);
    }

    /**
     * Shortcut to the catalog that holds the market's purchases which have not yet arrived.
     */
    public static COpenPurchaseOrders getOpenPurchaseOrder() {
        return (COpenPurchaseOrders)getTheShop().getCatalog(CAT_OPENPURCHASE);
    }

    /**
     * Shortcut to the catalog that holds all customers of the market.
     * This catalog is needed as a base for till- and warehouse-queue.
     */
    public static Catalog getCustomers(){
        return Shop.getTheShop().getCatalog(SMarket.CAT_CUSTOMER);
    }

    /**
     * Shortcut for the queue of customers who want to pay.
     */
    public static SSListenable getTillQueue(){
        return (SSListenable)Shop.getTheShop().getStock(SMarket.STK_TILLQUEUE);
    }

    /**
     * Shortcut for the queue of orders waiting at the warehouse.
     */
    public static SSListenable getWarehouseQueue(){
        return (SSListenable)Shop.getTheShop().getStock(SMarket.STK_WAREHOUSEQUEUE);
    }

    /**
     * Shortcut to the catalog that holds the current day's statistics.
     */
    public static CSalesStats getDailySalesStats() {
        return (CSalesStats)(getTheMarket()).c_dailyStats;
    }

    /**
     * Shortcut to the catalog that holds the current month's statistics.
     */
    public static CSalesStats getMonthlySalesStats() {
        return (CSalesStats)(getTheMarket()).c_monthlyStats;
    }

    /**
     * Shortcut to the statistics catalog
     */
    public static CCompleteStats getCompleteSalesStats() {
        return (CCompleteStats)getTheShop().getCatalog(CAT_COMPLETE_SALESSTATS);
    }

    /**
     * Shortcut to the customer stats catalog
     */
    public static CCustomerStats getCustomerStats() {
        return getTheMarket().c_customerStats;
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Misc.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Returns Categories of all available Articles.
     * This method is used by dropdown filters for initialization.
     */
    public static Object[] getArticleCategories(){
        List l = new ArrayList();
        l.add("Alles");
        Iterator it = getArticleCatalog().iterator(null, false);
        CIArticle art = null;
        String category = null;
        while (it.hasNext()) {
            art = (CIArticle)it.next();
            category = art.getCategory();
            if (!l.contains(category)) {
                l.add(category);
            }
        }
        return l.toArray();
    }

    /**
     * Sets the market's state.
     *
     * @param i <ul>
     *             <li>0: The market is open, customers can buy</li>
     *             <li>1: The market is about to close, customers cannot login nor start new
     *                      purchases, even if they are already logged in.</li>
     *             <li>2: The market is closed, only the manager can log in</li>
     *          </ul>
     */
    public void setOpen(int i) {
        open = i;
        if (i == 0) {
            timeAdvanced = false;
            msiNewCustomer.setEnabled(true);
            msiCustomer.setEnabled(true);
            msiSeller.setEnabled(true);
            msiWorker.setEnabled(true);
        }
        if (i == 1) {
            fireMarketClosing();
            msiNewCustomer.setEnabled(false);
            msiCustomer.setEnabled(false);
        }
        if (i == 2) {
            fireMarketClosed();
            addDailyStatsToMonthlyStats();
            msiNewCustomer.setEnabled(false);
            msiCustomer.setEnabled(false);
            msiSeller.setEnabled(false);
            msiWorker.setEnabled(false);
        }
    }

    /**
     * @return whether the market is currently open or not.
     */
     public static boolean isOpen() {
        return getTheMarket().open == 0 || getTheMarket().open == 1;
     }

     /**
      * @return whether the manager has announced the market's closing or not.
      */
     public static boolean isToBeClosed() {
        return getTheMarket().open == 1;
     }

     /**
      * @return whether the time has advanced or not.
      * @see #timeAdvanced
      */
     public static boolean hasTimeAdvanced() {
        return getTheMarket().timeAdvanced;
     }


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Time
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

     /**
      * Convenience method
      *
      * @return the current date.
      */
    public static Calendar getTime() {
        return (Calendar)getTheMarket().getTimer().getTime();
    }

    /**
     * @return the date that the market has started its business.
     */
    public static Calendar getDateOfOpening() {
        return getTheMarket().dateOfOpening;
    }

    /**
     * Convencience method.
     *
     * @return the current year.
     */
    public static int getYear() {
        return getTime().get(Calendar.YEAR);
    }

    /**
     * Convenience method.
     *
     * @return the current month.
     */
    public static int getMonth() {
        return getTime().get(Calendar.MONTH);
    }

    /**
     * Sets the market's time to a new date.
     *
     * @param newTime the new date to be set.
     */
    public static void setTime(Calendar newTime) {
        int diff = Conversions.dayDifference(getTime(), newTime);
        getTheShop().getTimer().setTime(newTime);
        getTheMarket().computeEventsOnDayChanges(diff);
    }

    /**
     * Advances the market's time by 1 day.
     */
    public static void setNextDay() {
        getTheShop().getTimer().goAhead();
        getTheMarket().computeEventsOnDayChanges(1);
    }

    /**
     * Does things that need to be done when the time advances.
     * The Shop's title is set to the new date, the waiting time for new orders is decreased,
     * if new orders arrived they are added to the current offer.
     * If not only the day but also the month changed, the statistics and the market's account
     * are updated.
     *
     * @param daysAdvanced The number of days the time advanced.
     */
    private void computeEventsOnDayChanges(int daysAdvanced) {
        setShopFrameTitle(SHOP_CAPTION + "  -  " + getTime());
        timeAdvanced = true;
        //when open purchase orders arrive, add them to market's stock, otherwise just decrease the
        //numbers of days to wait
        CountingStock arrivedOrders = getOpenPurchaseOrder().subtractPassedDays(daysAdvanced);
        if(arrivedOrders.size(null)>0){
            getOffer().addStock(arrivedOrders, null, false);
            getWarehouseQueue().fireWakeUpOrders();
        }
        if (hasMonthChanged(daysAdvanced)) {//if month changed
            getMonthlySalesStats().setWages(
                    UMUserBase.getGlobalBase().getCurrentWages()); //save current wages to statistics
            getCompleteSalesStats().add(getMonthlySalesStats(), null); //update statistics catalogs
            int currentMiscCosts = c_monthlyStats.getCosts();
            considerJumpedOverMonths();
            c_monthlyStats = new CSalesStats(getYear(), getMonth());
            c_monthlyStats.setCosts(currentMiscCosts);
            c_monthlyStats.initPriceHistory();
            subtractFromAccount(new IntegerValue(monthlyCosts()));      //update account
        }
        fireTimeAdvanced();
    }

    /**
     * This method checks if the month changed due to the progress of time.
     *
     * @param daysAdvanced The number of days the time advanced.
     * @return true, if the month changed
     */
    private boolean hasMonthChanged(int daysAdvanced) {
        int dayOfMonth = getTime().get(Calendar.DAY_OF_MONTH);
        return daysAdvanced >= dayOfMonth;
    }

    /**
     * Adds an empty statistics entry to the statistics for every month that has been jumped over.
     * Besides, the monthly costs of the market are subtracted from the account.
     */
    private void considerJumpedOverMonths() {
        int currentMiscCosts = c_monthlyStats.getCosts();
        CCompleteStats ccs = getCompleteSalesStats();
        int m = Statistics.getLastArticleStatisticsMonth() + 1; //set not to last month, but to its successor
        int y = Statistics.getLastArticleStatisticsYear();
        if (m == 12) {
            m = 0;
            y++;
        }
        int mNow = getMonth();
        int yNow = getYear();
        while (m != mNow || y != yNow) {
            CSalesStats css = new CSalesStats(y, m);
            css.setCosts(currentMiscCosts);
            css.initPriceHistory();
            css.setWages(UMUserBase.getGlobalBase().getCurrentWages());
            ccs.add(css, null);
            subtractFromAccount(new IntegerValue(monthlyCosts()));      //update account
            m++;
            if (m == 12) {
                m = 0;
                y++;
            }
        }
    }

    /**
     * Adds today's statistic entries to the monthly stats.
     * This covers the sales statistics of every single article and the current day's total revenue.
     */
    private void addDailyStatsToMonthlyStats() {
        Iterator it = c_dailyStats.iterator(null, false);
        while (it.hasNext()) {
            CISalesStats dailyCiss = (CISalesStats)it.next();
            c_monthlyStats.get(dailyCiss.getArticleID()).addAmount(dailyCiss.getAmount());
            c_monthlyStats.get(dailyCiss.getArticleID()).addRevenue(dailyCiss.getRevenue());
        }
        c_monthlyStats.addRevenue(c_dailyStats.getRevenue());
        c_dailyStats = new CSalesStats(getYear(), getMonth());
    }

    /**
     * Convenience method that returns the total costs per month.
     * This covers both the whole wages and the miscellaneous costs that were set in the options.
     */
    private int monthlyCosts() {
        return UMUserBase.getGlobalBase().getCurrentWages() + c_monthlyStats.getCosts();
    }


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Event handling
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Adds a new EventListener to the market.
     */
    public static void addEventListener(MarketEventListener e) {
        MarketEventListener[] evl = (getTheMarket()).evl;
        int len = evl.length;
        boolean exists = false;
        for (int i = 0; i < len; i++) {
            exists = exists || (evl[i] == e);
        }
        if (!exists) {
            MarketEventListener[] temp = new MarketEventListener[len+1];
            System.arraycopy(evl, 0, temp, 0, len);
            temp[len] = e;
            evl = temp;
        }
        (getTheMarket()).evl = evl;
    }

    /**
     * Removes an EventListener from the market.
     */
    public static void removeEventListener(MarketEventListener e) {
        MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] == e) {
                MarketEventListener[] temp = new MarketEventListener[evl.length-1];
                if (i > 0) System.arraycopy(evl,0,temp,0,i);
                if (i < evl.length-1) System.arraycopy(evl,i+1,temp,i,evl.length-1-i);
                evl = temp;
                break;
            }
        }
        (getTheMarket()).evl = evl;
    }


    /**
     * Fires an event to all listeners: The market is about to close. (Which means, the manager pressed
     * his &quot;Closing-Time&quot; button.)
     */
    public static void fireMarketClosing()  {
        MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] != null) evl[i].notifyOnMarketClosing();
        }
    }

    /**
     * Fires an event to all listeners: The market isn't about to close anymore. (Which means, the manager
     * canceled closing time.)
     */
    public static void fireMarketNotClosing()  {
        MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] != null) evl[i].notifyOnMarketNotClosing();
        }
    }

    /**
     * Fires an event to all listeners: The market has just closed.
     */
    public static void fireMarketClosed()  {
        MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] != null) evl[i].marketClosed();
        }
    }

    /**
     * Fires an event to all listeners: The market has just opened.
     */
    public static void fireMarketOpened()  {
        MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] != null) evl[i].marketOpened();
        }
    }

    /**
     * Fires an event to all listeners: The time has advanced..
     */
    public static void fireTimeAdvanced()  {
        MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] != null) evl[i].timeAdvanced();
        }
    }

    /**
     * Fires an event to all listeners: The number of workers or orders to process has changed.
     */
    public static void fireUpdateWorkerScreen(){
         MarketEventListener[] evl = (getTheMarket()).evl;
        for (int i = 0; i < evl.length; i++) {
            if (evl[i] != null) evl[i].workerInformationChanged();
        }
    }
}
