package market;

import java.awt.Rectangle;

import market.event.OfferEventListener;
import market.stdform.ButtonIDs;
import market.stdform.FSCustomerDefault;
import market.stdform.MSLogOff;
import sale.Action;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.MenuSheet;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import users.User;
import data.CountingStock;
import data.DataBasketCondition;
import data.DataBasketEntry;
import data.DataBasketEntryDestination;
import data.DataBasketEntrySource;

/**
 * SalesPoint used by the {@link UCustomer}.
 */
public class SPCustomer extends SPListenable implements OfferEventListener{

    /**
     * Array with all SPCustomers used to fire events to
     */
    private static OfferEventListener[] oel = new OfferEventListener[0];

    /**
     * @param user the User of this SPCustomer
     */
    public SPCustomer(User user){
        super("Kundenterminal - "+((UPerson)user).getFullName());
        this.attach(user);
        this.setSalesPointFrameBounds(new Rectangle(0,0,640,540));
        SMarket.addEventListener((market.event.MarketEventListener)this);
        SPCustomer.addEventListener(this);
        Shop.getTheShop().addSalesPoint(this);
    }


    //###################################### SalesPoint #############################################

    /**
     * @return <code>true</code> if no SaleProcess is running, otherwise <code>false</code>.
     * This forces the user to quit the SaleProcess before he closes the SalesPoint.
     */
    protected boolean onCanQuit(){
        return getCurrentProcess() == null;
    }

    /**
     * Removes this SPSustomer from the array of OfferEventListeners.
     */
    public void quit() {
        SPCustomer.removeEventListener(this);
        super.quit();
    }

    /**
     * @return the default FormSheet
     */
    protected FormSheet getDefaultFormSheet() {
        FormSheet sheet = new FSCustomerDefault();
        sheet.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                fs.getButton(ButtonIDs.BTN_BUY).setAction(buyAction());
                fs.getButton(ButtonIDs.BTN_EDIT).setAction(editAction());
            }
        });
        return sheet;
    }

    /**
     * @return the default MenuSheet
     */
    protected MenuSheet getDefaultMenuSheet() {
        return new MSLogOff();
    }


    //################################### our methods ######################################################

    /**
     * @return an Action that initiates a SProcessBuy on this Salespoint,
     * if the customer already contains to the till-queue or the market is to be closed
     * it will pop up an error-message.
     */
    private Action buyAction(){
        return new Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                User user = sp.getUser();
                boolean error = false;
                String errorMsg = "";
                String errorCaption = "";
                if (SMarket.isToBeClosed()) {
                    error = true;
                    errorMsg = SMarket.MARKET_CLOSES_LONG;
                    errorCaption = "Feierabend";
                }
                if(SMarket.getTillQueue().contains(user.getName(), null)) {
                    error = true;
                    errorMsg = "Sie haben bereits eine Auswahl von Artikeln in Auftrag gegeben!\n"+
                            "Bitte begeben Sie sich zunchst zur Kasse, um diesen Autrag abzuschliessen!";
                    errorCaption = "Auftrag abschlieen";
                }
                if (error) {
                    JDDShowMessage.showMessageDialog(sp.getDisplay().getFormSheet(), errorMsg,
                           errorCaption);
                } else {
                    sp.runProcess(new SProcessCustomer(user));
                }
            }
        };
    }

    /**
     * @return an Action that initiates a SProcessCustomerEditProfile on this SalesPoint,
     * taking the customer attached to this SPCustomer as argument.
     */
    private Action editAction(){
        return new Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                boolean error = false;
                String errorMsg = "";
                String errorCaption = "";
                if (SMarket.isToBeClosed()) {
                    error = true;
                    errorMsg = SMarket.MARKET_CLOSES_LONG;
                    errorCaption = "Feierabend";
                }
                if (error) {
                    JDDShowMessage.showMessageDialog(sp.getDisplay().getFormSheet(), errorMsg,
                           errorCaption);
                } else {
                    sp.runProcess(new SProcessCustomerEditProfile(sp.getUser()));
                }
            }
        };
    }


    ////////////////////////////////////////////////////////////////
    // Event handling
    ////////////////////////////////////////////////////////////////

    /**
     * Adds an OfferEventListener to the array of listeners.
     *
     * @param e the OfferEventListener that will be added.
     */
    public static void addEventListener(OfferEventListener e){
        int len = oel.length;
        boolean exists = false;
        for (int i = 0; i < len; i++) {
            exists = exists || (oel[i] == e);
        }
        if (!exists) {
            OfferEventListener[] temp = new OfferEventListener[len+1];
            System.arraycopy(oel, 0, temp, 0, len);
            temp[len] = e;
            oel = temp;
        }
    }

    /**
     * Removes an OfferEventListener from the array of listeners.
     *
     * @param e the OfferEventListener that will be removed.
     */
    public static void removeEventListener(OfferEventListener e){
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] == e) {
                OfferEventListener[] temp = new OfferEventListener[oel.length-1];
                if (i > 0) System.arraycopy(oel,0,temp,0,i);
                if (i < oel.length-1) System.arraycopy(oel,i+1,temp,i,oel.length-1-i);
                oel = temp;
                break;
            }
        }
    }

    /**
     * Fires an event to all listeners: this article is empty.
     *
     * @param articleKey the key of the unavailable article.
     */
    public static void fireOfferIsEmpty(String articleKey) {
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] != null) oel[i].offerEmpty(articleKey);
        }
    }

    /**
     * Fires an event to all listeners: count this article.
     *
     * @param articleKey the key of the article to count.
     * @param spw the SProcessWorker which has sended the request.
     */
    public static void fireCountArticles(String articleKey, SProcessWorker spw){
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] != null) oel[i].countArticles(articleKey, spw);
        }
    }

    /**
     * Reaction on event: An article is unavailable.
     *
     * @param articleKey the unavailable article.
     */
    public void offerEmpty(final String articleKey) {
        CountingStock cs = ((UCustomer)this.getUser()).getShoppingBasket();
        if(this.getCurrentProcess() instanceof SProcessCustomer &&
           cs.contains(articleKey, this.getBasket())){
            this.getBasket().rollback(new DataBasketCondition(){
            public String getMainKey() {
                return null;
            }
            public String getSecondaryKey() {
                return null;
            }
            public DataBasketEntrySource getSource() {
                return null;
            }
            public DataBasketEntryDestination getDestination() {
                return null;
            }
            public Object getValue() {
                return null;
            }
            public boolean match(DataBasketEntry dbe) {
                if(dbe.getSecondaryKey().compareTo(articleKey)==0) return true;
                return false;
            }
            });
            try {
                this.getCurrentProcess().suspend();
            } catch (InterruptedException e) {
            }
            JDDShowMessage.showMessageDialog(getDisplay().getFormSheet(),
                        "Aufgrund einer Bestandskorrektur musste folgender Artikel: "+
                        SMarket.getArticleCatalog().get(articleKey).getArticleName()+
                        " aus ihrem Einkaufskorb entfernt werden.\n" +
                        "Bitte whlen sie neu.", "Artikel nicht verfgbar! "+
                        ((UPerson)this.getUser()).getFullName());
        }
    }

    /**
     * Empty implementation of the OfferEventListener Interface.
     */
    public void wakeUpOrders() {
    }

    /**
     * Reaction on event: a SProcessWorker needs the count of all existing articles.
     *
     * @param articleKey the name of the article.
     * @param spw the SProcessWorker that sends the request.
     */
    public void countArticles(String articleKey, SProcessWorker spw) {
        int count = 0;
        if(this.getCurrentProcess() instanceof SProcessCustomer &&
            ((UCustomer)this.getUser()).getShoppingBasket().contains(articleKey, this.getBasket())){
            count = ((UCustomer)this.getUser()).getShoppingBasket().countItems(articleKey, this.getBasket());
            spw.addDatabaseCount(count, SProcessWorker.BUYPROCESS);
        }
    }
}