package market;

import java.util.Calendar;

import market.statistics.CISalesStats;
import market.statistics.Statistics;
import market.stdform.ButtonIDs;
import market.stdform.FSCheckable;
import market.stdform.FSEditPersonData;
import market.stdform.FSManagerArticleStatsDetail;
import market.stdform.FSManagerArticleStatsMain;
import market.stdform.FSManagerCustomerOverview;
import market.stdform.FSManagerCustomerStatsDetail;
import market.stdform.FSManagerCustomerStatsMain;
import market.stdform.FSManagerEmployeeOverview;
import market.stdform.FSManagerOffer;
import market.stdform.FSManagerOpenClose;
import market.stdform.FSManagerOpenPurchaseOrderDetail;
import market.stdform.FSManagerOpenPurchaseOrderMain;
import market.stdform.FSManagerOptions;
import market.stdform.FSManagerOverallStats;
import market.stdform.FSManagerPurchase;
import market.stdform.FSManagerPurchaseConfirm;
import market.stdform.MSLogOff;
import market.swing.JCTimeRangeBoxes;
import sale.Action;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.Gate;
import sale.GateChangeTransition;
import sale.JDisplayDialog;
import sale.MenuSheet;
import sale.MenuSheetItem;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import sale.Transition;
import sale.UIGate;
import data.CatalogItemValue;
import data.CountingStock;
import data.DataBasket;
import data.DataBasketConditionImpl;
import data.DoubleValue;
import data.IntegerValue;
import data.Value;
import data.events.VetoException;
import data.ooimpl.CountingStockImpl;
import data.ooimpl.DataBasketImpl;
import data.stdforms.SingleTableFormSheet;
import data.stdforms.TwoTableFormSheet;

/**
 * The manager process. This process handles all interaction with the manager's {@link SalesPoint}.
 */
public class SProcessManager extends SProcessMarket {

    /**
     * Gate for opening and closing the market.
     */
    private UIGate gateOpenClose                = new UIGate(null, null);

    /**
     * Gate for setting some calculation variables.
     */
    private UIGate gateOptions                  = new UIGate(null, null);

    /**
     * Gate for purchasing some articles for the market.
     */
    private UIGate gatePurchaseMain             = new UIGate(null, null);

    /**
     * Gate for confirming a purchase.
     */
    private UIGate gatePurchaseConfirm          = new UIGate(null, null);

    /**
     * Gate for displaying an overview of orders that have not yet arrived.
     */
    private UIGate gateOpoMain                  = new UIGate(null, null);

    /**
     * Gate for displaying details of an that has not yet arrived.
     */
    private UIGate gateOpoDetail                = new UIGate(null, null);

    /**
     * Gate for displaying the market's articles and their amount.
     */
    private UIGate gateStockMain                = new UIGate(null, null);

    /**
     * Gate for viewing, selecting, adding and removing employees.
     */
    private UIGate gateEmplEditMain             = new UIGate(null, null);

    /**
     * Gate for editing an employee's data.
     */
    private UIGate gateEmplEditDetail           = new UIGate(null, null);

    /**
     * Gate for viewing, selecting and removing customers.
     */
    private UIGate gateCustEditMain             = new UIGate(null, null);

    /**
     * Gate for viewing a customer's personal data.
     */
    private UIGate gateCustEditDetail           = new UIGate(null, null);

    /**
     * Gate for selecting an article and a range of time for statistics.
     */
    private UIGate gateArticleStatsMain         = new UIGate(null, null);

    /**
     * Gate for viewing detailed article statistics.
     */
    private UIGate gateArticleStatsDetail       = new UIGate(null, null);

    /**
     * Gate for selecting a customer whose statistics should be displayed.
     */
    private UIGate gateCustomerStatsMain        = new UIGate(null, null);

    /**
     * Gate for viewing a customer's statistics.
     */
    private UIGate gateCustomerStatsDetail      = new UIGate(null, null);

    /**
     * Gate for the overall statistics.
     */
    private UIGate gateOverallStats             = new UIGate(null, null);
    private MenuSheet menu = createManagerMenuSheet();
    private DataBasket dbPurchase;
    private CountingStock csPurchase;

    /**
     * The last date the manger ordered something. This variable necessary for creating the indexes of
     * the open purchase orders.
     */
    private Calendar dateLastOrder;

    /**
     * A counting variable for open purchase orders. If there are more than one orders a day, they are
     * distinguished by a running number which is created with the help of this variable.
     */
    private int intLastOrder = 0;

    public SProcessManager() {
        super("Manager");
    }

    /**
     * This method is needed by {@link SaleProcess} to know on which gate to start.
     * @return the initial gate, i.e. the {@link Gate} where the process starts.
     */
    public Gate getInitialGate() {
        return getStockMainGate();
    }

    /**
     * Attaches {@link FSManagerOpenClose}, its actions and the menu to {@link #gateOpenClose}.
     * @return the set up {@link #gateOpenClose}.
     */
    private UIGate getOpenCloseGate() {
        final FormSheet fsmoc = new FSManagerOpenClose();
        fsmoc.addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                //open market
                fs.getButton(ButtonIDs.BTN_START).setAction(new Action() {
                    public void doAction(SaleProcess p, SalesPoint sp) {
                        SMarket.getTheMarket().setOpen(0);
                        SMarket.fireMarketOpened();
                    }
                });
                //close market (end of day)
                fs.getButton(ButtonIDs.BTN_END).setAction(new Action() {
                    public void doAction(SaleProcess p, SalesPoint sp) {
                        int error = 0;
                        if (Shop.getTheShop().getSalesPoints().size() > 1) error = 1;
                        if (SMarket.getTillQueue().size(null) > 0) error = 2;
                        switch (error) {
                            case 1:
                                JDDShowMessage.showMessageDialog(fs,
                                      "Es sind noch nicht alle SalesPoints geschlossen", "Fehler");
                                break;

                            case 2:
                                JDDShowMessage.showMessageDialog(fs,
                                      "Es wurden noch nicht alle Kunden bedient.", "Fehler");
                                break;
                            default:
                                SMarket.getTheMarket().setOpen(2);
                        }
                    }
                });
                //next day
                fs.getButton(ButtonIDs.BTN_NEXT).setAction(
                    new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                          SMarket.setNextDay();
                    }
                });
            }
        });

        //open market
        setAction(fsmoc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    SMarket.getTheMarket().setOpen(0);
                    SMarket.fireMarketOpened();
                }}, ButtonIDs.BTN_START);
        //close market
        setAction(fsmoc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    int error = 0;
                    if (Shop.getTheShop().getSalesPoints().size() > 1) error = 1;
                    if (SMarket.getTheMarket().getStock(SMarket.STK_TILLQUEUE).size(null) > 0) error = 2;
                    switch (error) {
                        case 1:
                            JDDShowMessage.showMessageDialog(fsmoc,
                                  "Es sind noch nicht alle SalesPoints geschlossen", "Fehler");
                            break;

                        case 2:
                            JDDShowMessage.showMessageDialog(fsmoc,
                                  "Es wurden noch nicht alle Kunden bedient.", "Fehler");
                            break;
                        default:
                            SMarket.getTheMarket().setOpen(2);
                    }
                }}, ButtonIDs.BTN_END);
        //advance time
        setAction(fsmoc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    SMarket.setNextDay();
                }}, ButtonIDs.BTN_NEXT);

        gateOpenClose.setFormSheet(fsmoc);
        gateOpenClose.setMenuSheet(menu);
        return gateOpenClose;
    }

    /**
     * Attaches {@link FSManagerOptions}, its actions and the menu to {@link #gateOptions}.
     * @return the set up {@link #gateOptions}.
     */
    private UIGate getOptionsGate() {
        final FSCheckable fsc = new FSCheckable(new FSManagerOptions());
        //accept settings
        setAction(fsc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    if (fsc.checkTextFields(FSCheckable.ALL_ERRORMESSAGES_AT_ONCE, false)) {
                        Options options = SMarket.getOptions();
                        int error = 0;
                        if (Double.valueOf(Conversions.convertComma(
                                fsc.getEntry(FSManagerOptions.JTFC_MAX_DISCOUNT))).doubleValue() > 100)
                            error = 1;
                        switch (error) {
                            case 1:
                                JDDShowMessage.showMessageDialog(fsc, "Es knnen nicht mehr als " +
                                        "100% Rabatt gewhrt werden.", "Zu viel Rabatt");
                            default:
                                options.setDiscountRange(Integer.valueOf(
                                        fsc.getEntry(FSManagerOptions.JTFC_DISCOUNT_RANGE)).intValue());
                                options.setDiscountValue(Integer.valueOf(
                                        fsc.getEntry(FSManagerOptions.JTFC_DISCOUNT_VALUE)).intValue());
                                options.setMaxDiscount(Double.valueOf(Conversions.convertComma(
                                        fsc.getEntry(FSManagerOptions.JTFC_MAX_DISCOUNT))).
                                        doubleValue() / 100);
                                options.setFractionOfWages(Double.valueOf(Conversions.convertComma(
                                        fsc.getEntry(FSManagerOptions.JTFC_FRACTION_OF_WAGES))).
                                        doubleValue() / 100);
                                options.setTimeOfEmployment(Double.valueOf(Conversions.convertComma(
                                        fsc.getEntry(FSManagerOptions.JTFC_SENIORITY))).
                                        doubleValue() / 100);
                                SMarket.getMonthlySalesStats().setCosts(Conversions.currencyToInt(
                                        fsc.getEntry(FSManagerOptions.JTFC_COSTS), true));
                        }
                    }
                }}, ButtonIDs.BTN_ACCEPT);
        gateOptions.setFormSheet(fsc);
        gateOptions.setMenuSheet(menu);
        return gateOptions;
    }

    /**
     * Attaches {@link FSManagerOffer} and the menu to {@link #gateStockMain}.
     * @return the set up {@link #gateStockMain}.
     */
    private UIGate getStockMainGate() {
        final SingleTableFormSheet stfs = FSManagerOffer.create();
        gateStockMain.setFormSheet(stfs);
        gateStockMain.setMenuSheet(menu);
        return gateStockMain;
    }

    /**
     * Attaches {@link FSManagerPurchase}, its actions and the menu to {@link #gatePurchaseMain}.
     * @param cs the manager's "shopping basket", it can be empty or already filled with articles.
     * @param db the transaction's DataBasket.
     * @return the set up {@link #gatePurchaseMain}.
     */
    private UIGate getPurchaseMainGate(CountingStock cs, DataBasket db) {
        //Create FormSheet with saved DataBasket and CountingStock if needed. If not, create new ones.
        final TwoTableFormSheet ttfs = FSManagerPurchase.create(gatePurchaseMain,
                cs == null ? new CountingStockImpl("Einkauf", SMarket.getArticleCatalog()) : cs,
                db == null ? new DataBasketImpl() : db);
        //Buy
        setAction(ttfs, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    csPurchase = (CountingStock)ttfs.getRightTableSource();
                    dbPurchase = ttfs.getDataBasket();
                    //if no items bought, print error message
                    if (!dbPurchase.contains(DataBasketConditionImpl.ALL_ENTRIES)) {
                        JDDShowMessage.showMessageDialog(ttfs, "Bitte whlen sie Artikel zum Kauf aus.",
                                "Keine Auswahl getroffen");
                    } else {
                        //sum up prices of all selected Articles
                        DoubleValue dv = new DoubleValue(0);
                        csPurchase.sumStock(dbPurchase, CatalogItemValue.EVALUATE_OFFER, dv);
                        int toPay = Conversions.currencyToInt(dv.toString(), false);
                        //Transition to gate purchaseConfirm
                        gatePurchaseMain.setNextTransition(new GateChangeTransition(
                                getPurchaseConfirmGate(toPay)));
                    }
                }}, ButtonIDs.BTN_BUY);
        gatePurchaseMain.setFormSheet(ttfs);
        gatePurchaseMain.setMenuSheet(menu);
        return gatePurchaseMain;
    }

    /**
     * Attaches {@link FSManagerPurchaseConfirm} and its actions to {@link #gatePurchaseConfirm}.
     * @param toPay the money to pay for the manager's purchase.
     * @return the set up {@link #gatePurchaseConfirm}.
     */
    private UIGate getPurchaseConfirmGate(final int toPay) {
        final FormSheet fsmpc = new FSManagerPurchaseConfirm(toPay);
        //buy (commit db, add cs to offerStock) and go back to purchaseMain with right table being empty
        setAction(fsmpc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    dbPurchase.commit();
                    //create id for new order (current date and consecutive number)
                    Calendar now = SMarket.getTime();
                    if (dateLastOrder == null) {
                        dateLastOrder = (Calendar)now.clone();
                    }
                    if (dateLastOrder.before(now)) {
                        dateLastOrder = (Calendar)now.clone();
                        intLastOrder = 1;
                    } else {
                        intLastOrder++;
                    }
                    String newIDDate = dateLastOrder.toString();
                    CIOpenPurchaseOrders opo = new CIOpenPurchaseOrders(newIDDate, intLastOrder,
                            csPurchase);
                    SMarket.getOpenPurchaseOrder().add(opo, null);
                    //add this order to statistics
                    SMarket.getMonthlySalesStats().updateOrderHistory(opo.getOrders());
                    gatePurchaseConfirm.setNextTransition(new GateChangeTransition(
                            getPurchaseMainGate(null, null)));
                    //subtract purchase from account
                    SMarket.subtractFromAccount(new IntegerValue(toPay));
                }}, ButtonIDs.BTN_OK);
        //go back to purchaseMain and find right table as it has been left so one can edit his purchase
        setAction(fsmpc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    ((UIGate)p.getCurrentGate()).setNextTransition(
                            new GateChangeTransition(getPurchaseMainGate(csPurchase, dbPurchase)));
                }}, ButtonIDs.BTN_BACK);
        //cancel purchase and go back to purchaseMain with right table being empty
        setAction(fsmpc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    dbPurchase.rollback();
                    gatePurchaseConfirm.setNextTransition(new GateChangeTransition(
                            getPurchaseMainGate(csPurchase, dbPurchase)));
                }}, ButtonIDs.BTN_CANCEL);
        gatePurchaseConfirm.setFormSheet(fsmpc);
        return gatePurchaseConfirm;
    }

    /**
     * Attaches {@link FSManagerOpenPurchaseOrderMain}, its actions and the menu to {@link #gateOpoMain}.
     * @return the set up {@link #gateOpoMain}.
     */
    private UIGate getOpoMainGate() {
        final SingleTableFormSheet stfs = FSManagerOpenPurchaseOrderMain.create();
        //view details of the selected order
        setAction(stfs, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    CIOpenPurchaseOrders selRec = (CIOpenPurchaseOrders)stfs.getSelectedRecord();
                    if (selRec == null)
                        JDDShowMessage.showMessageDialog(stfs, "Bitte whlen sie eine Lieferung aus.",
                                "Auswahl treffen");
                    else
                        ((UIGate)p.getCurrentGate()).setNextTransition(new GateChangeTransition(
                                getOpoDetailGate(selRec)));
                }}, ButtonIDs.BTN_DETAIL);
        gateOpoMain.setFormSheet(stfs);
        gateOpoMain.setMenuSheet(menu);
        return gateOpoMain;
    }

    /**
     * Attaches {@link FSManagerOpenPurchaseOrderDetail}, its actions and the menu to {@link #gateOpoDetail}.
     * @param opo the open purchase order to be viewed.
     * @return the set up {@link #gateOpoDetail}.
     */
    private UIGate getOpoDetailGate(CIOpenPurchaseOrders opo) {
        FormSheet fsmopod = FSManagerOpenPurchaseOrderDetail.create(opo);
        //go back to overview
        setTransition(fsmopod, new GateChangeTransition(getOpoMainGate()), ButtonIDs.BTN_BACK);
        gateOpoDetail.setFormSheet(fsmopod);
        gateOpoDetail.setMenuSheet(menu);
        return gateOpoDetail;
    }


    /**
     * Attaches {@link FSManagerEmployeeOverview}, its actions and the menu to {@link #gateEmplEditMain}.
     * @return the set up {@link #gateEmplEditMain}.
     */
    private UIGate getEmplEditMainGate() {
        final FSManagerEmployeeOverview fsmeo = new FSManagerEmployeeOverview();
        //button add (merely a transition, but defined as action because of circular reference)
        setAction(fsmeo, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    ((UIGate)p.getCurrentGate()).setNextTransition(
                            new GateChangeTransition(getEmplEditDetailGate(null)));
                }}, ButtonIDs.BTN_ADD);
        //button edit
        setAction(fsmeo, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    UStaffer usr = (UStaffer)fsmeo.getSelectedRecord();
                    if (usr == null) {
                        JDDShowMessage.showMessageDialog(fsmeo, "Bitte whlen sie einen Mitarbeiter aus.",
                                "Auswahl treffen");
                    } else {
                        ((UIGate)p.getCurrentGate()).setNextTransition(
                                new GateChangeTransition(getEmplEditDetailGate(usr)));
                    }
                }}, ButtonIDs.BTN_EDIT);
        //button delete
        setAction(fsmeo, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    final UStaffer usr = (UStaffer)fsmeo.getSelectedRecord();
                    if (usr == null) {
                        JDDShowMessage.showMessageDialog(fsmeo, "Bitte whlen sie einen Mitarbeiter aus.",
                                "Auswahl treffen");
                    } else {
                        final Value dismissalCompensation = usr.computeDismissalCompensation();
                        JDisplayDialog jdd = JDDShowMessage.showMessageDialog(fsmeo,
                                "Soll " + usr.getFullName() + " ein Entlassungsgeld in Hhe von " +
                                Conversions.valueToCurrency(dismissalCompensation, " Euro") +
                                " gezahlt werden?", "Entlassungsgeld");
                        jdd.getFormSheet().addContentCreator(new FormSheetContentCreator() {
                            protected void createFormSheetContent(final FormSheet fs) {
                                fs.removeAllButtons();
                                fs.addButton("Ja", 1, new Action() {
                                    public void doAction(SaleProcess p, SalesPoint sp) {
                                        try {
                                            UMUserBase.deleteUser(usr);
                                            SMarket.subtractFromAccount(dismissalCompensation);
                                        }
                                        catch (VetoException ex) {
                                            JDDShowMessage.showMessageDialog(fsmeo, ex.getMessage(),
                                                    "Eingeloggt");
                                        }
                                        fs.cancel();
                                    }
                                });
                                fs.addButton("Nein", 2, new Action() {
                                    public void doAction(SaleProcess p, SalesPoint sp) {
                                        try {
                                            UMUserBase.deleteUser(usr);
                                        }
                                        catch (VetoException ex) {
                                            JDDShowMessage.showMessageDialog(fsmeo, ex.getMessage(),
                                                    "Eingeloggt");
                                        }
                                        fs.cancel();
                                    }
                                });
                                fs.addButton("Abbrechen", 3, new Action() {
                                    public void doAction(SaleProcess p, SalesPoint sp) {
                                        fs.cancel();
                                    }
                                });

                            }

                        });
                    }
                }}, ButtonIDs.BTN_DELETE);
        gateEmplEditMain.setFormSheet(fsmeo);
        gateEmplEditMain.setMenuSheet(menu);

        return gateEmplEditMain;
    }

    /**
     * Attaches {@link FSEditPersonData}, its actions and the menu to {@link #gateEmplEditDetail}.
     * @param usr the employee whose details are to be viewed or edited. If <code>null</code> the
     * FormSheet's fields will be empty and a new worker can be employed.
     * @return the set up {@link #gateEmplEditDetail}.
     */
    private UIGate getEmplEditDetailGate(final UStaffer usr) {
        final FSCheckable fsc = FSEditPersonData.getStafferProfile(usr);
        //button accept
        setAction(fsc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    if (fsc.checkTextFields(FSCheckable.ALL_ERRORMESSAGES_AT_ONCE, false)) {
                        FSEditPersonData fsepd = (FSEditPersonData)fsc.getFormSheet();
                        char[] pwd = fsepd.getPassword();
                        //check vor special errors
                        int error = 0;
                        if (usr == null) { //if new user is to be added
                            if (pwd == null)
                                error = 1; //the passwords must be set
                        } else { //if old user is to be edited
                            if (UMUserBase.getNumberOfManagers() == 1   //last manager mustn't be removed
                                    && usr.getQualification().equals(UStaffer.MANAGER)
                                    && !fsepd.getQualification().equals(UStaffer.MANAGER))
                                error = 2;  //password does not have to be set, but password and confirmation
                                            //must at least match
                            if (!fsepd.passwordsEqual())
                                error = 3;
                        }
                        switch (error) {
                            case 1:
                                JDDShowMessage.showMessageDialog(fsc,
                                        "Die Passwrter stimmen nicht berein oder wurden nicht gesetzt.",
                                        "Passwort berprfen");
                                break;
                            case 2:
                                JDDShowMessage.showMessageDialog(fsc,
                                        "Dies ist der letzte Manager, er darf seines Postens nicht " +
                                        "enthoben werden.",
                                        "Letzter Manager");
                                break;
                            case 3:
                                JDDShowMessage.showMessageDialog(fsc,
                                        "Die Passwrter stimmen nicht berein.",
                                        "Passwort berprfen");
                                break;
                        }
                        if (error == 0) {
                            UStaffer u = null;
                            if (usr == null) { //new user
                                u = (UStaffer)UMUserBase.createUser(fsc.getEntry(FSEditPersonData.
                                        JTFC_LOGIN),
                                        UMUserBase.SELLER,
                                        fsepd.getQualification());
                            } else { //existing user
                                u = usr;
                            }
                            if (u == null) { //new user, but chosen login already exists
                                JDDShowMessage.showMessageDialog(fsc,
                                        "Dieses Login wurde bereits vergeben, whlen sie bitte ein anderes",
                                        "Doppeltes Login");
                            } else {
                                //both
                                u.setSalutation(fsepd.getSalutation());
                                u.setSurname(fsc.getEntry(FSEditPersonData.JTFC_NAME));
                                u.setFirstName(fsc.getEntry(FSEditPersonData.JTFC_FIRSTNAME));
                                u.setTelephone(fsc.getEntry(FSEditPersonData.JTFC_TELEPHONE));
                                u.setStreet(fsc.getEntry(FSEditPersonData.JTFC_STREET));
                                u.setCity(fsc.getEntry(FSEditPersonData.JTFC_CITY));
                                u.setPostcode(new Integer(
                                        fsc.getEntry(FSEditPersonData.JTFC_POSTCODE)).intValue());
                                if (fsepd.isPasswordSet())
                                    u.setPassWd(UStaffer.garblePassWD(pwd));
                                u.setQualification(fsepd.getQualification());
                                u.setSalary(Conversions.currencyToInt(
                                        fsc.getEntry(FSEditPersonData.JTFC_SALARY), true));
                                ((UIGate)p.getCurrentGate()).setNextTransition(
                                        new GateChangeTransition(getEmplEditMainGate()));
                            }
                        }
                    }
                }}, ButtonIDs.BTN_ACCEPT);

        //button back (merely a transition, but defined as action because of circular reference)
        setAction(fsc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    ((UIGate)p.getCurrentGate()).setNextTransition(
                            new GateChangeTransition(getEmplEditMainGate()));
                }}, ButtonIDs.BTN_BACK);
        gateEmplEditDetail.setFormSheet(fsc);
        gateEmplEditDetail.setMenuSheet(menu);
        return gateEmplEditDetail;
    }

    /**
     * Attaches {@link FSManagerCustomerOverview}, its actions and the menu to {@link #gateCustEditMain}.
     * @return the set up {@link #gateCustEditMain}.
     */
    private UIGate getCustEditMainGate() {
        final FSManagerCustomerOverview fsmco = new FSManagerCustomerOverview();
        //button edit
        setAction(fsmco, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    UCustomer usr = (UCustomer)fsmco.getSelectedRecord();
                    if (usr == null) {
                        JDDShowMessage.showMessageDialog(fsmco, "Bitte whlen sie einen Kunden aus.",
                                "Auswahl treffen");
                    } else {
                        ((UIGate)p.getCurrentGate()).setNextTransition(
                                new GateChangeTransition(getCustEditDetailGate(usr)));
                    }
                }}, ButtonIDs.BTN_EDIT);
        //button delete
        setAction(fsmco, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    UCustomer usr = (UCustomer)fsmco.getSelectedRecord();
                    if (usr == null) {
                        JDDShowMessage.showMessageDialog(fsmco, "Bitte whlen sie einen Kunden aus.",
                                "Auswahl treffen");
                    } else {
                        try {
                            UMUserBase.deleteUser(usr);
                        }
                        catch (VetoException ex) {
                            JDDShowMessage.showMessageDialog(fsmco,
                                    ex.getMessage(), "Eingeloggt");
                        }
                    }
                }}, ButtonIDs.BTN_DELETE);
        gateCustEditMain.setFormSheet(fsmco);
        gateCustEditMain.setMenuSheet(menu);
        return gateCustEditMain;
    }

    /**
     * Attaches {@link FSEditPersonData}, its actions and the menu to {@link #gateCustEditDetail}.
     * @param usr the customer whose details are to be viewed.
     * @return the set up {@link #gateCustEditDetail}.
     */
    private UIGate getCustEditDetailGate(final UCustomer usr) {
        final FSCheckable fsc = FSEditPersonData.getCustomerProfileForManager(usr);
        //button accept
        setAction(fsc, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    if (fsc.checkTextFields(FSCheckable.ALL_ERRORMESSAGES_AT_ONCE, false)) {
                        FSEditPersonData fsepd = (FSEditPersonData)fsc.getFormSheet();
                        char[] pwd = fsepd.getPassword();
                        //password and confirmation do not match
                        if (!fsepd.passwordsEqual()) {
                            JDDShowMessage.showMessageDialog(fsc,
                                    "Die Passwrter stimmen nicht berein.",
                                    "Passwort berprfen");
                        } else {
                            //save changes
                            if (fsepd.isPasswordSet()) usr.setPassWd(UStaffer.garblePassWD(pwd));
                            ((UIGate)p.getCurrentGate()).setNextTransition(
                                    new GateChangeTransition(getCustEditMainGate()));
                        }
                    }
                }}, ButtonIDs.BTN_ACCEPT);
        //button back
        setTransition(fsc, new GateChangeTransition(getCustEditMainGate()), ButtonIDs.BTN_BACK);
        fsc.getButton(ButtonIDs.BTN_BACK).setCaption("Kundenbersicht");
        //button to stats
        fsc.addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                fsc.addButton("Zur Statistik", 1, new Action() {
                    public void doAction(SaleProcess p, SalesPoint sp) {
                        ((UIGate)p.getCurrentGate()).setNextTransition(
                                new GateChangeTransition(getCustomerStatsDetailGate(usr)));
                    }
                });
            }
        });
        gateCustEditDetail.setFormSheet(fsc);
        gateCustEditDetail.setMenuSheet(menu);
        return gateCustEditDetail;
    }

    /**
     * Attaches {@link FSManagerArticleStatsMain}, its actions and the menu to {@link #gateArticleStatsMain}.
     * @return the set up {@link #gateArticleStatsMain}.
     */
    private UIGate getArticleStatsMainGate(){
        //instantiation necessary to query ComboBoxes
        FSManagerArticleStatsMain fsmasm = new FSManagerArticleStatsMain();
        final JCTimeRangeBoxes jctrb = fsmasm.getTimeRangeBoxes();
        final SingleTableFormSheet stfs = FSManagerArticleStatsMain.create();
        //button detail
        setAction(stfs, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    CISalesStats statsToDisplay = null;
                    int error = 0;
                    CIArticle selRec = (CIArticle)stfs.getSelectedRecord();
                    if (selRec == null) error = 1;
                    if (!jctrb.isValidTimeRange()) error = 2;
                    switch (error) {
                        case 1:
                            JDDShowMessage.showMessageDialog(stfs, "Bitte whlen sie einen Artikel aus.",
                                "Auswahl treffen");
                            break;
                        case 2:
                            JDDShowMessage.showMessageDialog(stfs, "Ungltiger Zeitraum.",
                                   "Ungltiger Zeitraum");
                            break;
                        default:
                            int mFrom = jctrb.getFromMonth();
                            int mTo = jctrb.getToMonth();
                            int yFrom = jctrb.getFromYear();
                            int yTo = jctrb.getToYear();
                            ((UIGate)p.getCurrentGate()).setNextTransition(new GateChangeTransition(
                                    getArticleStatsDetailGate(Statistics.getArticleStats(
                                    selRec.getName(), mFrom, yFrom, mTo, yTo))));
                    }
                }}, ButtonIDs.BTN_DETAIL);
        gateArticleStatsMain.setFormSheet(stfs);
        gateArticleStatsMain.setMenuSheet(menu);
        return gateArticleStatsMain;
    }

    /**
     * Attaches {@link FSManagerArticleStatsDetail}, its actions and the menu to
     * {@link #gateArticleStatsDetail}.
     * @param ciss the precomputed statistics to be displayed.
     * @return the set up {@link #gateArticleStatsDetail}.
     */
    public Gate getArticleStatsDetailGate(CISalesStats ciss) {
        FormSheet fsasd = new FSManagerArticleStatsDetail(ciss);
        //button back
        setTransition(fsasd, new GateChangeTransition(getArticleStatsMainGate()), ButtonIDs.BTN_BACK);
        gateArticleStatsDetail.setFormSheet(fsasd);
        gateArticleStatsDetail.setMenuSheet(menu);
        return gateArticleStatsDetail;
    }

    /**
     * Attaches {@link FSManagerCustomerStatsMain}, its actions and the menu to {@link #gateCustomerStatsMain}.
     * @return the set up {@link #gateCustomerStatsMain}.
     */
    private UIGate getCustomerStatsMainGate() {
        final FSManagerCustomerStatsMain fsmcsm = new FSManagerCustomerStatsMain();
        //button detail
        setAction(fsmcsm, new Action() {public void doAction(SaleProcess p, SalesPoint sp) {
                    UCustomer selRec = (UCustomer)fsmcsm.getSelectedRecord();
                    int error = 0;
                    if (selRec == null) error = 1;
                    switch (error) {
                        case 1:
                            JDDShowMessage.showMessageDialog(fsmcsm, "Bitte whlen sie einen Kunden aus.",
                                "Auswahl treffen");
                            break;
                        default:
                            ((UIGate)p.getCurrentGate()).setNextTransition(new GateChangeTransition(
                                    getCustomerStatsDetailGate(selRec)));
                    }
                }}, ButtonIDs.BTN_DETAIL);
        gateCustomerStatsMain.setFormSheet(fsmcsm);
        gateCustomerStatsMain.setMenuSheet(menu);
        return gateCustomerStatsMain;
    }

    /**
     * Attaches {@link FSManagerCustomerStatsDetail}, its actions and the menu to
     * {@link #gateCustomerStatsDetail}.
     * @param uc the customer whose statistics are to be viewed.
     * @return the set up {@link #gateCustomerStatsDetail}.
     */
    private UIGate getCustomerStatsDetailGate(final UCustomer uc) {
        final FormSheet fscsd = new FSManagerCustomerStatsDetail(uc);
        fscsd.addContentCreator(new FormSheetContentCreator() {
            public void createFormSheetContent(final FormSheet fs) {
                //button to customer data
                fs.addButton("Kundendaten", 1, new Action() {
                    public void doAction(SaleProcess p, SalesPoint sp) {
                        ((UIGate)p.getCurrentGate()).setNextTransition(
                                new GateChangeTransition(getCustEditDetailGate(uc)));
                    }
                });
            }
        });
        setTransition(fscsd, new GateChangeTransition(getCustomerStatsMainGate()), ButtonIDs.BTN_BACK);
        fscsd.getButton(ButtonIDs.BTN_BACK).setCaption("Statistikbersicht");
        gateCustomerStatsDetail.setFormSheet(fscsd);
        gateCustomerStatsDetail.setMenuSheet(menu);
        return gateCustomerStatsDetail;
    }

    /**
     * Attaches {@link FSManagerOverallStats} and the menu to {@link #gateOverallStats}.
     * @return the set up {@link #gateOverallStats}.
     */
    private UIGate getOverallStatsGate() {
        FormSheet fsmos = new FSManagerOverallStats();
        gateOverallStats.setFormSheet(fsmos);
        gateOverallStats.setMenuSheet(menu);
        return gateOverallStats;
    }

    /**
     * Creates the {@link MenuSheet} for the manager process.
     *
     * @return the created MenuSheet.
     */
    private MenuSheet createManagerMenuSheet() {
        MenuSheet msMenuBar = new MenuSheet("Bar");
        MenuSheet msLogOff = new MSLogOff();
        MenuSheet msWares = new MenuSheet("Waren", null, 'W');
        MenuSheet msPersons = new MenuSheet("Personen", null, 'P');
        MenuSheet msStatistics = new MenuSheet("Statistiken", null, 'k');
        MenuSheetItem msiOpenClose = new MenuSheetItem("ffnen/Schlieen", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getOpenCloseGate()));
            }}, 'f');
        MenuSheetItem msiOptions = new MenuSheetItem("Einstellungen", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getOptionsGate()));
            }}, 'E');
        MenuSheetItem msiStockOverview = new MenuSheetItem("Bestand und Preis", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(new GateChangeTransition(getStockMainGate()));
            }}, 'B');
        MenuSheetItem msiPurchase = new MenuSheetItem("Einkaufen", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(getToPurchaseMain(null, null));
            }}, 'E');
        MenuSheetItem msiOpoMain = new MenuSheetItem("Erwartete Lieferungen", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getOpoMainGate()));
            }}, 'L');
        MenuSheetItem msiEmplEdit = new MenuSheetItem("Angestellte", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getEmplEditMainGate()));
            }}, 'A');
        MenuSheetItem msiCustEdit = new MenuSheetItem("Kunden", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getCustEditMainGate()));
            }}, 'K');
        MenuSheetItem msiArticleStats = new MenuSheetItem("Artikelstatistik", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getArticleStatsMainGate()));
            }}, 'A');
        MenuSheetItem msiCustomerStats = new MenuSheetItem("Kundenstatistik", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getCustomerStatsMainGate()));
            }}, 'K');
        MenuSheetItem msiOverallStats = new MenuSheetItem("Umsatzstatistik", null, new Action() {
            public void doAction(SaleProcess p, SalesPoint sp) {
                ((UIGate)p.getCurrentGate()).setNextTransition(
                        new GateChangeTransition(getOverallStatsGate()));
            }}, 'U');

        msMenuBar.add(msLogOff);
            msLogOff.add(msiOpenClose);
            msLogOff.add(msiOptions);
            msLogOff.add(msLogOff.remove("exit"));
        msMenuBar.add(msWares);
            msWares.add(msiStockOverview);
            msWares.add(msiPurchase);
            msWares.add(msiOpoMain);
        msMenuBar.add(msPersons);
            msPersons.add(msiEmplEdit);
            msPersons.add(msiCustEdit);
        msMenuBar.add(msStatistics);
            msStatistics.add(msiArticleStats);
            msStatistics.add(msiCustomerStats);
            msStatistics.add(msiOverallStats);
        return msMenuBar;
    }

  /**
   * Helper Transition to {@link #gatePurchaseMain}.<br>
   * <br>
   * This transition cannot be implemented as a GateChangeTransition.<br>
   * The reason is that getPurchaseMainGate contains a TTFS-creation method that passes
   * gatePurchaseMain as necessary parameter for the TTFS.<br>
   * When a user's current Gate is gatePurchaseMain and he invokes a GateChangeTransition
   * to gatePurchaseMain via the menu, an exception would occur.<br>
   * This is because for a GateChangeTransition the setup method for gatePurchaseMain
   * (and therefore the TTFS-creation method) would be called while gatePurchaseMain
   * is still active and displaying a TTFS.
   * This method creates the TTFS after the gatePurchaseMain has already been
   * left, thus causing no conflicts.
   *
   * @param cs the CountingStock containing the manager's purchase.
   * @param db the transaction's DataBasket.
   *
   * @return a Transition to {@link #getPurchaseMainGate(CountingStock, DataBasket)}.
   */
    private Transition getToPurchaseMain(final CountingStock cs, final DataBasket db) {
        return new Transition() {
            public Gate perform(SaleProcess pOwner, users.User usr) {
                return getPurchaseMainGate(cs, db);
            }
        };
    }
}
