package market;

import sale.Action;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Transition;
import sale.UIGate;

/**
 * Helper class that supports the division of {@link FormSheet} and {@link SaleProcess}.<br>
 * <br>
 * Button actions which are attached to the FormSheets in the process are meant to be
 * reloaded after the shop's state has been saved. Therefore it is necessary to put the assignment of the
 * button actions to Buttons into a {@link FormSheetContentCreator}.<br>
 * As this assignment code is always the same, it has been put into this extra class. That makes the original
 * code more concise.
 */
public abstract class SProcessMarket extends SaleProcess {

    /**
     * @param name the name of the process.
     */
    public SProcessMarket(String name) {
        super(name);
    }

    /**
     * Assigns an {@link Action} to a FormSheet's button using a {@link FormSheetContentCreator}.
     *
     * @param formSheet the FormSheet to which the action should be assigned.
     * @param ac the action to be assigned.
     * @param btn_id the button ID to which the action should be assigned. A button with this
     * ID must be defined in the FormSheet itself or a {@link NullPointerException} will occur.
     */
    protected void setAction(FormSheet formSheet, final Action ac, final int btn_id){
        formSheet.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                fs.getButton(btn_id).setAction(ac);
            }
        });
    }

    /**
     * Assigns an {@link Action}, which consists only of a {@link Transition} to a FormSheet's
     * button using a {@link FormSheetContentCreator}.<br>
     * Caution has to be taken when a circular reference occurs.<br>
     * Example: Method getGateA() has a setTransition-method which changes to gateB using method
     * getGateB(). This method getGateB() contains a setTransition-method which changes to gateA using
     * the getGateA()-method. This causes an infinite loop.<br>
     * In contrast the {@link #setAction(FormSheet, Action, int) setAction()} method works well,
     * because the transition in it is not run on initialization, but only when the button has been pressed.
     *
     * @param formSheet the FormSheet from which the Transition starts.
     * @param trans the Transition to be performed.
     * @param btn_id the button ID that causes the transition to start. A button with this
     * ID must be defined in the FormSheet itself or a {@link NullPointerException} will occur.
     */
    protected void setTransition(FormSheet formSheet, final Transition trans, int btn_id){
        setAction(formSheet,
                new sale.Action(){
                    public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                        if(p.getCurrentGate()instanceof UIGate)
                            ((UIGate)p.getCurrentGate()).setNextTransition(trans);
                    }
                },
                btn_id);
    }
}