
package market;

import java.util.Hashtable;
import java.util.Iterator;

import market.stdform.ButtonIDs;
import market.stdform.FSCheckable;
import market.stdform.FSWorkerDefault;
import market.stdform.FSWorkerEdit;
import market.stdform.FSWorkerOrder;
import market.stdform.MSLogOff;
import sale.FormSheet;
import sale.Gate;
import sale.GateChangeTransition;
import sale.SaleProcess;
import sale.SalesPoint;
import sale.Shop;
import sale.Transition;
import sale.UIGate;
import sale.stdforms.MsgForm;
import users.User;
import users.UserManager;
import data.StockItem;
import data.events.VetoException;
import data.ooimpl.CountingStockImpl;
import data.stdforms.SingleTableFormSheet;

/**
 * The worker process. This process handles execution of the orders.
 */
public class SProcessWorker extends SProcessMarket{

    public static final int BUYPROCESS = 0;
    public static final int TILLQUEUE = 1;
    public static final int WAREHOUSEQUEUE = 2;
    public static final int OLDOFFER = 3;

    /**
     * Gate for displaying how much workers are currently logged on and
     * how much orders aren`t yet executed.
     */
    private UIGate uig_initial = new UIGate(null, null);

    /**
     * Gate for displaying a checklist for the order of the currently selected customer.
     */
    private UIGate uig_order = new UIGate(null, null);

    /**
     * Gate for displaying that the execution of the order isn`t completed.
     */
    private UIGate uig_notReady = new UIGate(null, null);

    /**
     * Gate for editing the count of articles in case of a shortage.
     */
    private UIGate uig_edit = new UIGate(null, null);

    /**
     * Gate for displaying that no article was selected.
     */
    private UIGate uig_noItemSelected = new UIGate(null, null);

    /**
     * Gate for affirming that the correction of the article`s count was successful.
     */
    private UIGate uig_editConfirmation = new UIGate(null, null);

    private SingleTableFormSheet stfs_order;
    private FSCheckable fsc_edit;

    /**
     * Current {@link CSOrder} to execute.
     */
    private CSOrder cso_order = null;

    /**
     * A new order that might contains articles which aren`t yet avaible in case of a shortage.
     */
    private CountingStockImpl cs_newOrder;

    /**
     * Current selected [@link SICustomer}.
     */
    private SICustomer sic_customer = null;

    /**
     * The global warehouse-queue.
     */
    private static SSListenable ss_orders = SMarket.getWarehouseQueue();

    /**
     * A Hashtable storing which articles are already executed and which not.
     */
    private Hashtable h_articlesDone = new Hashtable();

    /**
     * The {@link CIArticle} which count has to be edit.
     */
    private CIArticle ci_article = null;

    /**
     * An Array containing the count of an article in all different datastructures in the market,
     * temporarily items in shoppingbaskets, orders waiting at the till-queue,
     * orders waiting at the warehouse-queue and items in the markets-offer.
     */
    private int[] databaseCount =  new int[4];


    /**
     * @param sName the name of the process.
     */
    public SProcessWorker(String sName) {
        super(sName);
    }


    //################################### Gates ############################################################

    /**
     * Attaches {@link FSWorkerDefault}, its actions and the menu to {@link #uig_initial}.
     * @return the set up {@link #uig_initial}.
     */
    protected Gate getInitialGate() {
        FormSheet fs = new FSWorkerDefault(getOrderCount(), getWorkerCount());
        setTransition(fs, changeToOrderGate(), ButtonIDs.BTN_OK);
        uig_initial.setFormSheet(fs);
        uig_initial.setMenuSheet(new MSLogOff(logOff()));
        return uig_initial;
    }

    /**
     * Attaches {@link FSWorkerOrder} and its actions to {@link #uig_order}.
     * @return the set up {@link #uig_order}.
     */
    private Gate getOrderGate(){
        stfs_order = FSWorkerOrder.getOrderTable(sic_customer.getName(), cso_order, h_articlesDone);
        setTransition(stfs_order, GateChangeTransition.CHANGE_TO_COMMIT_GATE, ButtonIDs.BTN_ACCEPT);
        setTransition(stfs_order, changeToEditGate(), ButtonIDs.BTN_EDIT);
        setTransition(stfs_order, GateChangeTransition.CHANGE_TO_ROLLBACK_GATE, ButtonIDs.BTN_BACK);
        uig_order.setFormSheet(stfs_order);
        return uig_order;
    }

    /**
     * Attaches {@link MsgForm} and its ok-action to {@link #uig_notReady}.
     * @return the set up {@link #uig_notReady}.
     */
    private Gate notReadyGate(){
        FormSheet fs = new MsgForm("Auftrag nicht fertig","Sie haben nicht alle Artikel abgearbeitet.\n"+
        "Bitte erledigen Sie dies zunchst damit der Auftrag abgeschlossen werden kann!");
        setTransition(fs, new GateChangeTransition(uig_order), FormSheet.BTNID_OK);
        uig_notReady.setFormSheet(fs);
        return uig_notReady;
    }

    /**
     * Attaches {@link FSWorkerEdit} and its actions to {@link #uig_edit}.
     * @return the set up {@link #uig_edit}.
     */
    private Gate getEditGate(){
        fsc_edit = FSWorkerEdit.create(ci_article, getDatabaseCount());
        setAction(fsc_edit, new sale.Action(){
            public void doAction(SaleProcess p, SalesPoint sp) throws Throwable {
                if(fsc_edit.checkTextFields(FSCheckable.ALL_ERRORMESSAGES_AT_ONCE, false)){
                    uig_edit.setNextTransition(changeToEditConfirmationGate());
                }
            }
        }, ButtonIDs.BTN_OK);
        setTransition(fsc_edit, new GateChangeTransition(getOrderGate()), ButtonIDs.BTN_BACK);
        uig_edit.setFormSheet(fsc_edit);
        return uig_edit;
    }

    /**
     * Attaches {@link MsgForm} and its ok-action to {@link #uig_noItemSelected}.
     * @return the set up {@link #uig_noItemSelected}.
     */
    private Gate getNoItemSelectedGate(){
        FormSheet fs = new MsgForm("Kein Artikel gewhlt", "Sie mssen zunchst einen Artikel auswhlen!");
        setTransition(fs, new GateChangeTransition(getOrderGate()), FormSheet.BTNID_OK);
        uig_noItemSelected.setFormSheet(fs);
        return uig_noItemSelected;
    }

    /**
     * Attaches {@link MsgForm} and its ok-action to {@link #uig_editConfirmation}.
     * @return the set up {@link #uig_editConfirmation}.
     */
    private Gate getEditConfirmationGate(){
        FormSheet fs = new MsgForm("Bestand aktualisiert", "Der Bestand wurde erfolgreich korrigiert!");
        setTransition(fs, new GateChangeTransition(getOrderGate()), FormSheet.BTNID_OK);
        uig_editConfirmation.setFormSheet(fs);
        return uig_editConfirmation;
    }

    /**
     * @return the Gate to jump to if the current order is complete.
     */
    public Gate getCommitGate() {
        return new Gate(){
            public Transition getNextTransition(SaleProcess pOwner, User usr)
                    throws InterruptedException {
                return commit();
            }
        };
    }

    /**
     * @return the Gate to jump to if the current order has to be rolled back.
     */
    public Gate getRollbackGate() {
        return new Gate(){
            public Transition getNextTransition(SaleProcess pOwner, User usr)
                    throws InterruptedException {
                return rollback();
            }
        };
    }

    //################################## Transitions ##########################################################

    /**
     * @return a Transition that changes to the {@link #getOrderGate()} if there are any orders left,
     * otherwise returns back to {@link #getInitialGate()},
     * sets the next customer and order.
     */
    private Transition changeToOrderGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                setNextCustomer();
                if(sic_customer==null) return getInitialGate();
                try {
                    ss_orders.remove(sic_customer, pOwner.getBasket());
                } catch (VetoException e) {
                    System.err.println(e.getMessage());
                    return getInitialGate();
                }
                cso_order = sic_customer.removeOrderFromQueue();
                initiateArticlesDone();
                return getOrderGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getInitialGate()} if order is completed,
     * otherwise changes to the {@link #notReadyGate()},
     * adds a new CSOrder to the customer if some articles were not avaible.
     */
    private Transition commit(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                if(h_articlesDone.contains(Boolean.FALSE)) return notReadyGate();
                if(cs_newOrder != null) sic_customer.addOrderToQueue(cs_newOrder, false);
                if(sic_customer.getOrderCount(false)==0) getBasket().commit();
                else getBasket().rollback();
                Shop.getTheShop().removeStock(cso_order.getName());
                sic_customer = null;
                cso_order = null;
                cs_newOrder = null;
                h_articlesDone.clear();
                return getInitialGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getInitialGate()},
     * rolls back the previously selected order.
     */
    private Transition rollback(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                pOwner.getBasket().rollback();
                sic_customer.rollbackOrder(cso_order);
                sic_customer = null;
                cso_order = null;
                h_articlesDone.clear();
                ci_article = null;
                cs_newOrder = null;
                return getInitialGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getEditGate()} if an article was choosed,
     * otherwise changes to the {@link #getNoItemSelectedGate()}.
     */
    private Transition changeToEditGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                if(stfs_order.getSelectedRecord()==null) return getNoItemSelectedGate();
                ci_article = Conversions.recordToCIArticle(stfs_order.getSelectedRecord());
                countDatabase();
                return getEditGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getEditConfirmationGate()},
     * corrects the count of the currently selected article in market`s dates.
     */
    private Transition changeToEditConfirmationGate(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                // Checks whether the new number of articles is smaller than the old one
                // because if their is no shortage there is no need to update
                // this can be done during an inventure or something like this
                if(Integer.parseInt(fsc_edit.getEntry(FSWorkerEdit.JTFC_REAL))<=getDatabaseCount()){
                    updateOffer();
                }
                return getEditConfirmationGate();
            }
        };
    }

    /**
     * @return a Transition that changes to the {@link #getStopGate()},
     * fires Event: updateWorkerScreen to all MarketEventListeners
     */
    private Transition logOff(){
        return new Transition(){
            public Gate perform(SaleProcess pOwner, User usr) {
                SMarket.fireUpdateWorkerScreen();
                return getStopGate();
            }
        };
    }

    //#################################### internal methods #####################################################

    /**
     * @return the count of all workers which are currently logged on.
     */
    private static int getWorkerCount(){
        int worker = 0;
        Iterator salespoints = Shop.getTheShop().getSalesPoints().iterator();
        while(salespoints.hasNext()){
            User user = ((SalesPoint)salespoints.next()).getUser();
            if(((UMUserBase)UserManager.getGlobalUM()).getWarehouseWorker().match(user)){
                worker++;
            }
        }
        return worker;
    }

    /**
     * @return the count of all orders which aren`t completed yet.
     */
    private static int getOrderCount(){
        int order = 0;
        Iterator it = ss_orders.iterator(null, false);
        while(it.hasNext()){
            order = order + ((SICustomer)it.next()).getOrderCount(true);
        }
        return order;
    }

    /**
     * Selects the next customer and order to be execute.
     */
    private void setNextCustomer(){
        Long time = new Long(System.currentTimeMillis());
        SICustomer sic_next = null;
        Iterator it = ss_orders.iterator(null, true);
        while(it.hasNext()){
            SICustomer sic_current = (SICustomer)it.next();
            if((time.compareTo(sic_current.getOrderQueueTime())>=0) && (sic_current.getOrderCount(true) > 0)){
                time = sic_current.getOrderQueueTime();
                sic_next = sic_current;
            }
        }
        sic_customer = sic_next;
    }

    /**
     * Initiates the HashTable with false in the value-fields to signal the articles aren`t executed yet.
     */
    private void initiateArticlesDone(){
        Iterator it = cso_order.iterator(null, false);
        while(it.hasNext()){
            h_articlesDone.put(((StockItem)it.next()).getAssociatedItem(null), Boolean.FALSE);
        }
    }

    /**
     * @return the count of the selected article currently stored in the market`s database.
     */
    private int getDatabaseCount(){
        return databaseCount[BUYPROCESS]+
                databaseCount[TILLQUEUE]+
                databaseCount[WAREHOUSEQUEUE]+
                databaseCount[OLDOFFER];
    }

    /**
     * Adds a given value to the value at the specified position,
     * used by different implementations of OfferEventListener
     * to find out the count of an article considering all datastructures of the market.
     *
     * @param value the value that will be added.
     * @param type the index of the array at which the value will be added,
     * use {@link #BUYPROCESS} to {@link #OLDOFFER}.
     */
    public synchronized void addDatabaseCount(int value, int type){
        databaseCount[type] += value;
    }

    /**
     * Sets {@link #databaseCount} to the current count of the selected article,
     * uses Event: countArticles of OfferEventListener.
     */
    private void countDatabase(){
        databaseCount = new int[4];
        SPCustomer.fireCountArticles(ci_article.getName(), this);
        SMarket.getTillQueue().fireCountArticles(ci_article.getName(), this);
        SMarket.getWarehouseQueue().fireCountArticles(ci_article.getName(), this);
        addDatabaseCount(cso_order.countItems(ci_article.getName(), null), WAREHOUSEQUEUE);
        addDatabaseCount(SMarket.getOffer().countItems(ci_article.getName(), null), OLDOFFER);
    }

    /**
     * Updates the count of the selected article in market`s database,
     * if necessary fires Event: offerIsEmpty to OfferEventListeners.
     */
    private void updateOffer(){
        String aKey = ci_article.getName();
        int realCount = Integer.parseInt(fsc_edit.getEntry(FSWorkerEdit.JTFC_REAL));
        int newCount = databaseCount[OLDOFFER] + databaseCount[BUYPROCESS]
                    + databaseCount[TILLQUEUE] + databaseCount[WAREHOUSEQUEUE] - realCount;
        //if real count of the article doesn`t reach for orders at the till and warehouse
        //and selections of the customers, then first temporarily selected articles will be rolled back
        //by firing event: offerIsEmpty to {@link SPCustomer}
        if(realCount < (databaseCount[BUYPROCESS] + databaseCount[TILLQUEUE] + databaseCount[WAREHOUSEQUEUE])){
            SPCustomer.fireOfferIsEmpty(aKey);
            //if real count still doesn`t reach, the article will be removed from orders at the till-queue
            //by firing event: offerIsEmpty to {@link SICustomer}s at the till-queue
            if(realCount < (databaseCount[TILLQUEUE] + databaseCount[WAREHOUSEQUEUE])){
                SMarket.getTillQueue().fireOfferIsEmpty(aKey);
                //if real count still doesn`t reach, the article will be removed from orders
                //at the warehouse-queue,
                //by firing event: offerIsEmpty to {@link SICustomer}s at the warehouse-queue
                if(realCount<databaseCount[WAREHOUSEQUEUE]){
                    SMarket.getWarehouseQueue().fireOfferIsEmpty(aKey);
                    //if real count doesn`t reach for the current order,
                    //the missing one will be added as a new inactive order to the warehouse-queue
                    if(realCount < cso_order.countItems(aKey, null)){
                        int i = cso_order.countItems(aKey, null) - realCount;
                        cso_order.remove(aKey, i, null);
                        if(cs_newOrder == null){
                            cs_newOrder = new CountingStockImpl("new Order", SMarket.getArticleCatalog());
                        }
                        cs_newOrder.add(aKey, i, null);
                        if(!cso_order.contains(aKey, null)) h_articlesDone.remove(ci_article);
                        newCount -= i;
                    }
                }
            }

        }
        if(newCount <= 0) newCount = SMarket.getOffer().countItems(aKey, null);
        if(newCount > 0){
            try {
                SMarket.getOffer().remove(aKey, newCount, null);
            } catch (Exception e) {
                System.out.println(e.getMessage());
            }
        }
    }
}
