package market;

import market.event.OfferEventListener;
import data.DataBasket;
import data.DataBasketEntry;
import data.StockItem;
import data.ooimpl.CatalogImpl;
import data.ooimpl.StoringStockImpl;

/**
 * A StoringStock that can be used as a queue for tills and warehouse,
 * uses {@link SICustomer}s as StockItems and handles offer-events on it.
 */
public class SSListenable extends StoringStockImpl{

    /**
     * Array with all SPCustomers used to fire events to.
     */
    private OfferEventListener[] oel = new OfferEventListener[0];

    /**
     * @param sName the name of the new Stock.
     * @param ciRef the Catalog that is being referenced by the Stock.
     */
    public SSListenable(String sName, CatalogImpl ciRef) {
        super(sName, ciRef);
    }

    /**
     * Adds an OfferEventListener to the array of listeners.
     *
     * @param e the OfferEventListener that will be added.
     */
    public void addEventListener(OfferEventListener e){
        int len = oel.length;
        boolean exists = false;
        for (int i = 0; i < len; i++) {
            exists = exists || (oel[i] == e);
        }
        if (!exists) {
            OfferEventListener[] temp = new OfferEventListener[len+1];
            System.arraycopy(oel, 0, temp, 0, len);
            temp[len] = e;
            oel = temp;
        }
    }

    /**
     * Removes an OfferEventListener from the array of listeners.
     *
     * @param e the OfferEventListener that will be removed.
     */
    public void removeEventListener(OfferEventListener e){
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] == e) {
                OfferEventListener[] temp = new OfferEventListener[oel.length-1];
                if (i > 0) System.arraycopy(oel,0,temp,0,i);
                if (i < oel.length-1) System.arraycopy(oel,i+1,temp,i,oel.length-1-i);
                oel = temp;
                break;
            }
        }
    }

    /**
     * Fires an event to all listeners: this article is empty.
     *
     * @param articleKey the key of the unavaible article.
     */
    public void fireOfferIsEmpty(String articleKey) {
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] != null) oel[i].offerEmpty(articleKey);
        }
    }

    /**
     * Fires an event to all listeners: a delivery arrived at the market.
     */
    public void fireWakeUpOrders() {
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] != null) oel[i].wakeUpOrders();
        }
    }

    /**
     * Fires an event to all listeners: count this article.
     *
     * @param articleKey the key of the article to count.
     * @param spw the SProcessWorker which has sended the request.
     */
    public void fireCountArticles(String articleKey, SProcessWorker spw){
        for (int i = 0; i < oel.length; i++) {
            if (oel[i] != null) oel[i].countArticles(articleKey, spw);
        }
    }

    /**
     * Adds a StockItem to this Stock, and adds it to the array of listeners,
     * if it`s a SICustomer.
     *
     * @param si the item to be added.
     * @param db the DataBasket relative to which the item will be added.
     */
    public void add(StockItem si, DataBasket db) {
        if(si instanceof SICustomer)addEventListener(((SICustomer)si));
        super.add(si, db);
    }

    /**
    * Commit the removal of a StockItem, removes it from the array of listeners.
    */
    public void commitRemove(DataBasket db, DataBasketEntry dbe) {
        SICustomer sic = (SICustomer) dbe.getValue();
        removeEventListener(sic);
        super.commitRemove(db, dbe);
    }
}
