package market;

import java.util.Iterator;

import market.statistics.CICustomerStats;
import sale.SalesPoint;
import sale.Shop;
import users.User;
import users.UserManager;
import users.UserManagerFilter;
import users.swing.UserFilter;
import data.events.VetoException;

/**
 * The market's UserManager.
 */
public class UMUserBase extends UserManager {

    public static final int CUSTOMER = 1;
    public static final int WAREHOUSE_WORKER = 2;
    public static final int SELLER = 3;
    public static final int MANAGER = 4;

    /**
     * Filters market's customers.
     */
    private UserFilter ufCustomers = new UserFilter(){
        public boolean match(User user) {
            return UCustomer.class.isInstance(user);
        }
    };

    /**
     * Filters market's warehouse workers.
     */
    private UserFilter ufWarehouseWorker = new UserFilter(){
        public boolean match(User user) {
            if(UStaffer.class.isInstance(user) && ((UStaffer)user).getQualification().
                    equals(UStaffer.WAREHOUSE_WORKER)){
                return true;
            }
            return false;
        }
    };

    /**
     * Filters market's sellers.
     */
    private UserFilter ufSellers = new UserFilter(){
        public boolean match(User user) {
            if(UStaffer.class.isInstance(user) && ((UStaffer)user).getQualification().
                    equals(UStaffer.SELLER)){
                return true;
            }
            return false;
        }
    };

    /**
     * Filters the market's manager(s)
     */
    private UserFilter ufManager = new UserFilter(){
        public boolean match(User user) {
            if(UStaffer.class.isInstance(user) && ((UStaffer)user).getQualification().equals(UStaffer.MANAGER)){
                return true;
            }
            return false;
        }
    };

    /**
     * Filters the market's staff (warehouse-worker, seller, manager)
     */
    private UserFilter ufStaff = new UserFilter() {
                                    public boolean match(User user) {
                                        return user instanceof UStaffer;
                                    }
    };


    /**
     * A UserManagerFilter containing all managers of the market.
     */
    private UserManagerFilter umfManager = new UserManagerFilter(this, ufManager);


    /**
     * Static method that gets the {@link UserManager#getGlobalUM() global user manager}.
     * @return the global user manager.
     */
    public static UMUserBase getGlobalBase() {
        return (UMUserBase)getGlobalUM();
    }

    /**
     * creates a new User with given name, type and qualification
     *
     * @param sName the User's ID
     * @param type the User's type: 1 for customer, other numbers for employees.
     * @param qualification the employee's qualification (manager, seller, warehouse worker)
     */
    public static User createUser(String sName, int type, String qualification) {
        if(isUser(sName)) return null;
        User user;
        switch(type){
            case CUSTOMER: user = new UCustomer(sName);
                //add to customer catalog (base for queues)
                SMarket.getCustomers().add(Conversions.customerToCI((UCustomer)user), null);
                //add to customer statistics catalog
                SMarket.getCustomerStats().add(new CICustomerStats(user.getName()), null);
                break;
            default: user = new UStaffer(sName, qualification);
        }
        UserManager.getGlobalUM().addUser(user);
        return user;
    }

    /**
     * Checks the global UserManager for a String.
     *
     * @param userName the name of the user
     * @return <code>true</code> if a User could be identified by <code>userName</code>, otherwise
     * <code>false</code>.
     */
    public static boolean isUser(String userName) {
        if(UserManager.getGlobalUM().getUser(userName)==null){
            return false;
        }
        return true;
    }

    /**
     * Gets the {@link #ufCustomers customer filter}.
     * @return the customer filter.
     */
    public UserFilter getCustomers() {
        return ufCustomers;
    }

    /**
     * Gets the {@link #ufWarehouseWorker warehouse worker filter}.
     * @return the warehouse worker filter.
     */
    public UserFilter getWarehouseWorker() {
        return ufWarehouseWorker;
    }

    /**
     * Gets the {@link #ufSellers sellers filter}.
     * @return the sellers filter.
     */
    public UserFilter getSeller() {
        return ufSellers;
    }

    /**
     * Gets the {@link #ufManager mangaer filter}.
     * @return the manager filter.
     */
    public UserFilter getManager() {
        return ufManager;
    }

    /**
     * Gets the {@link #ufStaff staff filter}.
     * @return the staff filter.
     */
    public UserFilter getStaff() {
        return ufStaff;
    }

    /**
     * Counts and returns the number of Users that are managers.
     * @return the number of the market's managers.
     */
    public static int getNumberOfManagers() {
        return ((UMUserBase)getGlobalUM()).umfManager.getUsers().size();
    }

    /**
     * Checks whether there is a SalesPoint with the searched User logged on.
     *
     * @param user the searched user.
     * @return <code>true</code> if the user is logged on on any open SalesPoint,
     * otherwise <code>false</code>.
     */
    public static boolean isLoggedOn(User user){
        Iterator it = Shop.getTheShop().getSalesPoints().iterator();
        while(it.hasNext()){
            if(((SalesPoint)it.next()).getUser()==user) return true;
        }
        return false;
    }

    /**
     * Removes a User from market.
     *
     * @param user the user to be removed.
     * @throws VetoException if user to be removed is currently logged on.
     */
    public static void deleteUser(UPerson user) throws VetoException{
        if(!isLoggedOn(user)){
            String key = user.getName();
            if(user instanceof UCustomer) {
                SMarket.getOffer().addStock(((UCustomer)user).getShoppingBasket(), null, true);
                SSListenable orderQueue = SMarket.getWarehouseQueue();
                SSListenable tillQueue = SMarket.getTillQueue();
                if(orderQueue.contains(key, null)) orderQueue.remove(key, null);
                Iterator it = tillQueue.get(key, null, false);
                while(it.hasNext()){
                    if(((SICustomer)it.next()).getOrderCount(false)==0){
                        tillQueue.remove(key, null);
                    }
                }
                if(!tillQueue.contains(key, null)) SMarket.getCustomers().remove(key, null);
                SMarket.getCustomerStats().remove(key);
            }
            UserManager.getGlobalUM().deleteUser(key);
        }
        else {
            throw new VetoException("Nutzer " + user.getFullName() + " ist noch am System angemeldet.");
        }
    }

    /**
     * Sums up and returns wages of all currently employed persons.
     * @return the sum of all employee's wages.
     */
    public int getCurrentWages() {
        UserFilter staff = UMUserBase.getGlobalBase().getStaff();
        Iterator it = new UserManagerFilter(staff).getUsers().iterator();
        int sum = 0;
        while (it.hasNext()) {
            UStaffer us = (UStaffer)it.next();
            sum += Conversions.valueToInt(us.getSalary());
        }
        return sum;
    }
}

