package market;

import java.text.SimpleDateFormat;

/**
 * A ValueChecker, which checks if a String is in the format dd.MM.yyyy (e.g. 01.01.2000).<br>
 * Notations like 1.1.2000 are also not allowed.<br>
 * <br>
 * As this ValueChecker is only used by {@link market.swing.JTFCheckable checkable text fields},
 * an identifier is passed via the constructor. This descriptive identifier is used in the error string,
 * to enable the user to associate an error message with the causative text field.
 */
public class VCDate implements ValueChecker {

    /**
     * The identifier used by {@link #getErrorString()}.
     */
    protected String identifier;

    /**
     * Defines if an empty String is considered to be an error or not.
     */
    protected boolean mayBeEmpty;

    /**
     * Internal error code. Set by {@link #isValidValue(String)}, queried by {@link #getErrorString()}.
     */
    private int error = 0;

    /**
     * @param identifier the identifier.
     * @param mayBeEmpty <code>true</code> if an empty should be considered valid, otherwise <code>false</code>.
     */
    public VCDate(String identifier, boolean mayBeEmpty) {
        this.identifier = identifier;
        this.mayBeEmpty = mayBeEmpty;
    }

    /**
     * @param identifier the identifier.
     */
    public VCDate(String identifier) {
        this.identifier = identifier;
        this.mayBeEmpty = false;
    }

    /**
     * Checks String for validity and, if necessary, sets an internal error code according to the
     * detected error.
     * @param content the String to be checked.
     * @return <code>true</code> if the String is in a valid date format, otherwise <code>false</code>.
     */
    public boolean isValidValue(String content) {
        error = 0;
        if (!mayBeEmpty && content.equals("")) {
            error = 1;
        } else {
            //dd.MM.yyyy: strict check - the days' values are checked regarding the month, including leap years
            SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy");
            sdf.setLenient(false);
            try {
                sdf.parse(content);
                //Parsing also accepts dates without left handed zeros, so this extra check for the
                //String's length is being done
                if (content.length() != 10) {
                    error = 2;
                }
            }
            catch (java.text.ParseException e) {
                error = 2;
            }
        }
        return error == 0;
    }

    /**
     * @return an error message depending on the value of {@link #error}.
     */
    public String getErrorString() {
        String returnValue = "";
        switch (error) {
            case 1:
                returnValue = "Fehlerhafte Eingabe im Feld " + identifier +
                        ": Das Feld darf nicht leer sein.";
                break;
            case 2:
                returnValue = "Fehlerhafte Eingabe im Feld " + identifier +
                        ": Das Datum muss in der Form 01.01.2000 angegeben werden.";
                break;
        }
        error = 0;
        return returnValue;
    }
}