package market;

/**
 * A ValueChecker, which checks if a String is in the format of a Double (e.g. 15.256).<br>
 * Notations like 15,256 are also allowed. The number must not be negative.<br>
 * <br>
 * As this ValueChecker is only used by {@link market.swing.JTFCheckable checkable text fields},
 * an identifier is passed via the constructor. This descriptive identifier is used in the error string,
 * to enable the user to associate an error message with the causative text field.
 */
public class VCPositiveDouble implements ValueChecker {

    /**
     * The identifier used by {@link #getErrorString()}.
     */
    protected String identifier;

    /**
     * Defines if an empty String is considered to be an error or not.
     */
    protected boolean mayBeEmpty;

    /**
     * Internal error code. Set by {@link #isValidValue(String)}, queried by {@link #getErrorString()}.
     */
    private int error;

    /**
     * @param identifier the identifier.
     * @param mayBeEmpty <code>true</code> if an empty should be considered valid, otherwise <code>false</code>.
     */
    public VCPositiveDouble(String identifier, boolean mayBeEmpty) {
        this.identifier = identifier;
        this.mayBeEmpty = mayBeEmpty;
    }

    /**
     * @param identifier the identifier.
     */
    public VCPositiveDouble(String identifier) {
        this.identifier = identifier;
        this.mayBeEmpty = false;
    }

    /**
     * Checks String for validity and, if necessary, sets an internal error code according to the
     * detected error.
     * @param content the String to be checked.
     * @return <code>true</code> if the String represents a Double, otherwise <code>false</code>.
     */
    public boolean isValidValue(String content) {
        error = 0;
        if (!mayBeEmpty && content.equals("")) error = 1;
        if (!content.equals("") && error == 0) {
            if (Conversions.convertComma(content) == null) error = 2;
        }
        if (!content.equals("") && error == 0) {
            if (Double.valueOf(Conversions.convertComma(content)).doubleValue() < 0) error = 3;
        }
        return error == 0;
    }

    /**
     * @return an error message depending on the value of {@link #error}.
     */
    public String getErrorString() {
        String returnValue = "";
        switch (error) {
            case 1:  returnValue =  "Fehlerhafte Eingabe im Feld " + identifier + ": Das Feld darf nicht leer sein."; break;
            case 2:  returnValue = "Fehlerhafte Eingabe im Feld " + identifier + ": Es werden Kommazahlen erwartet."; break;
            case 3:  returnValue = "Fehlerhafte Eingabe im Feld " + identifier + ": Es werden positive Zahlen erwartet."; break;
        }
        error = 0;
        return returnValue;
    }
}