package market.statistics;

import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import market.SMarket;
import data.ooimpl.CatalogItemImpl;


/**
 * Represents a statistics for one article of the market's offer.
 */
public class CISalesStats extends CatalogItemImpl {

    private String articleID;
    private int revenue;
    private int amount;

    /**
     * Saves every change of the article's price. Both price and date of price change are recorded.
     */
    private List priceHistory = new LinkedList();

    /**
     * Saves the orders of the belonging item. Whenever an item is ordered, both the amount and the
     * date are recorded.
     */
    private List orderHistory = new LinkedList();

    /**
     * @param articleID the ID of the article statistics. It must match the real article's ID.
     * @param revenue the revenue made from this article.
     * @param amount the amount of sold items.
     */
    public CISalesStats(String articleID, int revenue, int amount) {
        super(articleID);
        this.articleID = articleID;
        this.revenue = revenue;
        this.amount = amount;
    }

    public String getArticleID() {
        return articleID;
    }

    public int getRevenue() {
        return revenue;
    }

    public int getAmount() {
        return amount;
    }

    /**
     * Returns the price history. If the last history entry is provisional, it is removed from
     * the returned list. It is, however, not removed from the internal {@link #priceHistory}.
     * @return the price history.
     */
    public List getPriceHistory() {
        List tmpPriceHistory = (List)((LinkedList)priceHistory).clone();
        if (isLastPriceChangeProvisional()) {
            tmpPriceHistory.remove(tmpPriceHistory.size() - 1);
        }
        return tmpPriceHistory;
    }

    /**
     * Checks if the last entry of the price history is provisional.
     * @return <code>true</code> if the last entry is provisional, otherwise <code>false</code>.
     */
    private boolean isLastPriceChangeProvisional() {
        int size = priceHistory.size();
        if (size == 0) {
            return false;
        }
        PriceHistoryEntry lastEntry = (PriceHistoryEntry)priceHistory.get(size-1);
        return lastEntry.isProvisional();
    }

    public List getOrderHistory() {
        return orderHistory;
    }

    /**
     * @param l the history ({@link #orderHistory} or {@link #priceHistory})list of which the
     * last item is of interest.
     * @return the last entry of a history list.
     */
    public HistoryEntry getLastEntry(List l) {
        int size = l.size();
        return size == 0 ? null : (HistoryEntry)l.get(size-1);
    }

    /**
     * Increases the saved amount of sold items.
     * @param amount the amount of itmes to add.
     */
    public void addAmount(int amount) {
        this.amount += amount;
    }

    /**
     * Increases the revenue made from this article.
     * @param revenue the revenue to be added.
     */
    public void addRevenue(int revenue) {
        this.revenue += revenue;
    }

    /**
     * Appends a new entry to the {@link #priceHistory}.
     * @param date the date of the price change.
     * @param newPrice the new price.
     */
    public void newPriceSet(Calendar date, int newPrice) {
        if (priceHistory.size() > 0) {
            PriceHistoryEntry lastEntry = (PriceHistoryEntry)priceHistory.get(priceHistory.size() - 1);
            if (lastEntry.isProvisional()) {
                priceHistory.remove(lastEntry);
            }
            if (!SMarket.hasTimeAdvanced()) {
                date.add(Calendar.DATE, 1);
            }
        }
        priceHistory.add(new PriceHistoryEntry(date, newPrice));
    }

    /**
     * Appends a new entry to the {@link #orderHistory}.
     * @param date the date of the price change.
     * @param amount the amount ordered
     */
    public void ordered(Calendar date, int amount) {
        orderHistory.add(new HistoryEntry(date, amount));
    }

    /**
     * Concatenates an external price history with this one.
     * @param ph the price history to be added.
     */
    public void appendPriceHistory(List ph) {
        priceHistory.addAll(ph);
    }

    /**
     * Concatenates an external order history with this one.
     * @param oh the order history to be added.
     */
    public void appendOrderHistory(List oh) {
        orderHistory.addAll(oh);
    }

    /**
     * Iterates over the order history and sums up the amount of bought items.
     * @return the amount of items ordered by the manager.
     */
    public int getOrderAmount() {
        int amount = 0;
        Iterator it = orderHistory.iterator();
        while (it.hasNext()) {
            amount += ((HistoryEntry)it.next()).getValue();
        }
        return amount;
    }

    public CatalogItemImpl getShallowClone() {
        CISalesStats ciss = new CISalesStats(articleID, revenue, amount);
        ciss.appendOrderHistory(orderHistory);
        ciss.appendPriceHistory(priceHistory);
        return ciss;
    }

    public String toString() {
        return "amount: " + amount + "; revenue: " + revenue + "; priceHistory: " +
                priceHistory + "; orderHistory: "  + orderHistory;
    }
}