package market.statistics;

import java.util.Calendar;
import java.util.Iterator;

import market.CIArticle;
import market.MarketCalendar;
import market.SMarket;
import market.UCustomer;
import data.CountingStock;
import data.events.VetoException;
import data.ooimpl.CatalogImpl;


/**
 * Represents a statistics of one month. It contains the statistics of every single offered
 * article and more general information such as paid wages and miscellaneous costs.
 * This class is also used as container in which overall statistics of a given range of time are brought
 * together.
 * The items of this catalog are {@link CISalesStats}.
 * @see Statistics#getOverallStats(int, int, int, int)
 */
public class CSalesStats extends CatalogImpl {

    private int month, year;
    private int costs, wages;
    private int revenue = 0;

    /**
     * Creates a new statistics container. To month that those statistics have been recorded is passed
     * via the parameters.
     * @param year the year in which the statistics were recorded.
     * @param month the month in which the statistics were recorded.
     */
    public CSalesStats(int year, int month) {
        super(year + "-" + (month < 10 ? "0" : "") + month);
        this.year = year;
        this.month = month;
        Iterator it = SMarket.getArticleCatalog().keySet(null).iterator();
        while (it.hasNext()) {
            String id = (String)it.next();
            CISalesStats ciss = new CISalesStats(id, 0, 0);
            add(ciss, null);
        }
    }

    /**
     * @return the statistic's year
     */
    public int getYear() {
        return year;
    }

    /**
     * @return the statistic's month
     */
    public int getMonth() {
        return month;
    }

    /**
     * Initializes the {@link CISalesStats#priceHistory price history} of each article with the current
     * article's bid. This initial bid is used for calculating average prices over a range of time.
     * Without it, it would be possible to have an empty price history for the chosen range of time which
     * makes calculations on this history more difficult.<br>
     * This method is called only on creation of a CCustomerStats, but it cannot be put into the
     * constructor, because there are cases when the initialization is not desired.
     * @see Statistics#getArticleStats(String, int, int, int, int)
     * @see Statistics#getOverallStats(int, int, int, int)
     * @see SMarket#c_dailyStats
     */
    public void initPriceHistory() {
        Iterator it = keySet(null).iterator();
        while (it.hasNext()) {
            String id = (String)it.next();
            CISalesStats ciss = get(id);
            Calendar c = new MarketCalendar(year, month, 1);
            if (c.before(SMarket.getDateOfOpening())) {
                c = SMarket.getDateOfOpening();
            }
            //set new price to first day of month or market's opening (only necessary for very first month)
            ciss.newPriceSet(c, SMarket.getArticleCatalog().get(id).getBid());
        }
    }

    /**
     * @param key the key of the CISalesStats to be returned.
     * @return the CISalesStats with the according key.
     */
    public CISalesStats get(String key) {
        try {
            return (CISalesStats)super.get(key, null, false);
        }
        catch (VetoException e) {
            return null;
        }
    }

    /**
     * Adds a customer's purchase to the article statistics.
     * @param customer the customer whose purchase is to be added.
     * @param discount the discount allowed by the market.
     */
    public void addSales(UCustomer customer, double discount) {
        CountingStock cs = customer.getShoppingBasket();
        Iterator it = cs.keySet(null).iterator();
        while (it.hasNext()) {
            String id = (String)it.next();
            CIArticle cia = SMarket.getArticleCatalog().get(id);
            int price = cia.getBid();
            int amount = cs.countItems(id, null);
            int revenue = new Double((amount*price)*(1-discount)).intValue();
            CISalesStats oldEntry = get(id);
            oldEntry.addAmount(amount);
            oldEntry.addRevenue(revenue);
        }
    }

    /**
     * Adds the current date to the order history of an article's statistics, whenever the manager
     * purchases it.
     * @see CISalesStats#ordered
     *
     * @param cs the manager's purchase.
     */
    public void updateOrderHistory(CountingStock cs) {
        Iterator it = cs.keySet(null).iterator();
        while (it.hasNext()) {
            String id = (String)it.next();
            int amount = cs.countItems(id, null);
            CISalesStats entry = get(id);
            entry.ordered(SMarket.getTime(), amount); //update order history
        }
    }

    /**
     * Adds the value customer's purchase to this month's revenue.
     * @param revenue the revenue to be added.
     */
    public void addRevenue(int revenue) {
        this.revenue += revenue;
    }

    /**
     * @return this month's revenue.
     */
    public int getRevenue() {
        return revenue;
    }

    /**
     * Sets the miscellaneous costs of the market. Only used while instance of this class represents
     * the current month.
     * @param costs the costs to be set.
     */
    public void setCosts(int costs) {
        this.costs = costs;
    }

    /**
     * @return this month's miscellaneous costs.
     */
    public int getCosts() {
        return costs;
    }

    /**
     * @return the costs of all article purchases the manager has made this month.
     */
    public int getOrderCosts() {
        Iterator it = keySet(null).iterator();
        int sum = 0;
        while (it.hasNext()) {
            String id = (String)it.next();
            CISalesStats ciss = get(id);
            sum += ciss.getOrderAmount() * SMarket.getArticleCatalog().get(id).getOffer();
        }
        return sum;
    }

    /**
     * Saves the sum of the employee's wages. Only used when month ends and statistics are saved to
     * {@link CCompleteStats}
     * @param wages the wages to be set.
     */
    public void setWages(int wages) {
        this.wages = wages;
    }

    /**
     * @return the saved wages. Will not work if instance of this class represents current month's statistics.
     */
    public int getWages() {
        return wages;
    }

    public String toString() {
        return "CSalesStats " + getName();
    }

    /**
     * Adds the all order histories which are stored in a different CSalesStats to this one.<br>
     * Using this method multiple times creates a complete order history for any desired range of time.
     * With the help of {@link #getOrderCosts()} it is now possible to easily compute the total of
     * money spent on purchases during that time.
     *
     * @param csToAdd the CSalesStats, which contains the order histories to be added.
     * @see Statistics#getOverallStats(int, int, int, int)
     */
    public void addOrders(CSalesStats csToAdd) {
        Iterator it = keySet(null).iterator();
        int sum = 0;
        while (it.hasNext()) {
            String id = (String)it.next();
            CISalesStats thisCiss = get(id);
            CISalesStats newCiss = csToAdd.get(id);
            thisCiss.appendOrderHistory(newCiss.getOrderHistory());
        }
    }
}
