package market.statistics;

import java.util.Calendar;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import market.Conversions;
import market.MarketCalendar;
import market.SMarket;

/**
 * Does calculation on statistics. While {@link Statistics#getArticleStats(String, int, int, int, int)}
 * sums up all prices and concatenates all history lists, this class {@link #cleanUpPriceHistory() cleans up}
 * the lists and provides methods to evaluate the statistics.
 */
public class EvaluateStatistics {

    private CISalesStats ciss;
    private List cleanedPriceHistory;
    private int averagePrice;
    private double averageOrder;
    private static final int MILLISECONDS_PER_DAY = 24 * 60 * 60 * 1000;

    /**
     * @param ciss the history item to be evaluated.
     */
    public EvaluateStatistics(CISalesStats ciss) {
        this.ciss = ciss;
        cleanedPriceHistory = cleanUpPriceHistory();
    }

    /**
     * Removes "duplicate" items from the {@link CISalesStats#priceHistory priceHistory}.<br>
     * As every price history is initialized with the current price on creation
     * (see {@link CSalesStats#initPriceHistory}), there might be multiple items in a row that contain
     * the same price.<br>
     * Another possibility to create consecutive items with the same price would be to set a new price in
     * the evening (after market closed), switch to the next day, set the price again to its original
     * value and change the day again (or open the market).
     * This method removes all list items that contain the same price as their predecessor.
     *
     * @return the cleaned up price history.
     */
    private List cleanUpPriceHistory() {
        List cleaned = new LinkedList();
        int lastValue = -1;
        List l = ciss.getPriceHistory();
        Iterator it = l.iterator();
        while (it.hasNext()) {
            HistoryEntry he = (HistoryEntry)it.next();
            if (he.getValue() != lastValue) {
                cleaned.add(he);
                lastValue = he.getValue();
            }
        }
        return cleaned;
    }

    /**
     * @return the date of the very first entry in the {@link #cleanUpPriceHistory cleaned up price history}.
     */
    private Calendar firstDate() {
        return ((HistoryEntry)cleanedPriceHistory.get(0)).getDate();
    }

    /**
     * @return the date of the very last entry in the {@link #cleanUpPriceHistory cleaned up price history}.
     */
    private Calendar lastRecordedDate() {
        return ((HistoryEntry)ciss.getPriceHistory().get(ciss.getPriceHistory().size() - 1)).getDate();
    }


    /**
     * Finds the last date of the time range for which the statistics should be computed (e.g. 31.01.2005).<br>
     * As every month has at least one item in the <b>original</b> price history, the last date can be
     * computed from that list.
     * @return the very last date of the statistic's time range.
     */
    private Calendar veryLastDate() {
        Calendar lastRecordedDate = lastRecordedDate();
        Calendar c = new MarketCalendar(lastRecordedDate.get(Calendar.YEAR),
                lastRecordedDate.get(Calendar.MONTH),
                lastRecordedDate.getActualMaximum(Calendar.DAY_OF_MONTH));
        //do not extrapolate to future, if last month is current month stop computation at day before current date
        if (!c.before(SMarket.getTime())) {
            c = new MarketCalendar(SMarket.getYear(), SMarket.getMonth(),
                    SMarket.getTime().get(Calendar.DATE) - 1);
        }
        return c;
    }

    /**
     * If market has just closed (i.e. day-end closing has taken place), take current day into account,
     * but if the market is open OR just the day has changed and the market hasn't
     * opened yet, ignore the current day.
     *
     * @return <code>true</code> when ignoring the current day for statistics, otherwise <code>false</code>.
     */
    private boolean ignoreCurrentDay() {
        return SMarket.isOpen() || SMarket.hasTimeAdvanced();
    }

    /**
     * @return the number of days covered by the statistics to be evaluated.
     */
    private int range() {
        int difference = Conversions.dayDifference(firstDate(), veryLastDate()) + 1;
        if (!ignoreCurrentDay()) {
            difference++;
        }
        return difference;
    }


    public List getPriceHistory() {
        return cleanedPriceHistory;
    }

    public List getOrderHistory() {
        return ciss.getOrderHistory();
    }

    /**
     * @return the average price of the article.
     */
    public int getAveragePrice() {
        int range = range();
        Iterator it = cleanedPriceHistory.iterator();
        HistoryEntry he = (HistoryEntry)it.next();
        int previousPrice = he.getValue();
        Calendar previousDate = he.getDate();
        int sum = 0;
        while (it.hasNext()) {
            he = (HistoryEntry)it.next();
            sum += previousPrice * Conversions.dayDifference(previousDate, he.getDate());
            previousDate = he.getDate();
            previousPrice = he.getValue();
        }
        sum += previousPrice * (Conversions.dayDifference(previousDate, veryLastDate()) +
                (ignoreCurrentDay() ? 1 : 2));
        //catch division by 0 if there are no days to be displayed yet (should only happen on date of opening)
        return (range == 0) ? SMarket.getArticleCatalog().get(ciss.getArticleID()).getBid() : sum/range;
    }

    public int getRevenue() {
        return ciss.getRevenue();
    }

    public int getAmount() {
        return ciss.getAmount();
    }

    /**
     * @return the average of sold items per day.
     */
    public double getAverageItemsSold() {
        int range = range();
        return range == 0 ? 0 : new Integer(getAmount()).doubleValue()/range;
    }

    /**
     * @return the average of items ordered per order.
     */
    public double getAverageOrderAmount() {
        double avg = 0;
        Iterator it = ciss.getOrderHistory().iterator();
        while (it.hasNext()) {
            avg += ((HistoryEntry)it.next()).getValue();
        }
        return ciss.getOrderHistory().size() == 0 ? 0 : avg/ciss.getOrderHistory().size();
    }

    /**
     * @return the average days between two orders of the appropriate item.
     */
    public double getAverageDaysBetweenOrders() {
        double avg = 0;
        Calendar prevOrder = null;
        Calendar nextOrder = null;
        Iterator it = ciss.getOrderHistory().iterator();
        if (it.hasNext()) {
            prevOrder = ((HistoryEntry)it.next()).getDate();
        }
        while (it.hasNext()) {
            nextOrder = ((HistoryEntry)it.next()).getDate();
            avg += Conversions.dayDifference(prevOrder, nextOrder);
            prevOrder = nextOrder;
        }
        if (prevOrder != null) {
            avg += Conversions.dayDifference(prevOrder, SMarket.getTime());
        }
        return prevOrder == null ? 0 : avg/ciss.getOrderHistory().size();
    }
}