package market.statistics;

import java.util.Calendar;
import java.util.Iterator;

import market.MarketCalendar;
import market.SMarket;
import market.UCustomer;
import market.UMUserBase;

/**
 * Helper class that handles the filtering and creating of analyzable statistics.
 */
public abstract class Statistics {

    public static Object[] MONTHS = new Object[] {"Januar", "Februar", "Mrz", "April", "Mai", "Juni",
            "Juli", "August", "September", "Oktober", "November", "Dezember"};

    /**
     * Creates an array of all years from which statistics are available.
     * This array is used by the {@link market.swing.JCTimeRangeBoxes JCTimeRangeBoxes} that let the user
     * select a time range for statistics to be displayed.
     *
     * @return the created array.
     */
    public static Object[] createArticleStatisticsYears() {
        int firstYear = getFirstArticleStatisticsYear();
        int lastYear = SMarket.getYear();
        int nrOfYears = lastYear - firstYear + 1;
        Object[] returnObject = new Object[nrOfYears];
        for (int i = 0; i < nrOfYears; i++) {
            returnObject[i] = new Integer(firstYear + i).toString();
        }
        return returnObject;
    }

    /**
     * Creates an array of all months of a given year from which statistics are available.
     * This array is used by the {@link market.swing.JCTimeRangeBoxes JCTimeRangeBoxes} that let the user
     * only select a time ranges for which statistics are available.
     *
     * @see market.swing.JCTimeRangeBoxes.FromItemListener
     * @see market.swing.JCTimeRangeBoxes.ToItemListener
     * @return the created array.
     */
    public static Object[] createArticleStatisticsMonths(Object yearString) {
        int year = new Integer((String)yearString).intValue();
        int firstMonth = 0;
        int lastMonth = 11;
        int firstYear = getFirstArticleStatisticsYear();
        int lastYear = getLastArticleStatisticsYear();
        if (getLastArticleStatisticsMonth() == 11) { //prevents returnObject from being initialized with a
            lastYear++;                              //negative number (happens if date set to Janary =>
        }                                            //firstMonth > lastMonth because firstYear equals
                                                     //lastYear and both firstMonth and lastMonth get new
                                                     //values)
        if (year == firstYear || year == lastYear) {
            if (year == firstYear) {
                firstMonth = getFirstArticleStatisticsMonth();
            }
            if (year == lastYear) {
                lastMonth = SMarket.getMonth();
            }
            Object[] returnObject = new Object[lastMonth - firstMonth + 1];
            System.arraycopy(MONTHS, firstMonth, returnObject, 0, returnObject.length);
            return returnObject;
        } else {
            return MONTHS;
        }
    }

    /**
     * Sums up a CIalesStats for the desired range of time.
     *
     * @param id the ID of the article for which statistics are asked for.
     * @param mFrom the first month of the statistics interval.
     * @param yFrom the first year of the statistics interval.
     * @param mTo the last month of the statistics interval.
     * @param yTo the last year of the statistics interval.
     *
     * @return the summed up CISalesStats.
     */
    public static CISalesStats getArticleStats(String id, int mFrom, int yFrom, int mTo, int yTo) {
        CISalesStats thisMonthsItem = SMarket.getMonthlySalesStats().get(id);
        CISalesStats returnStats = new CISalesStats(id, 0, 0);
        CCompleteStats ccs = SMarket.getCompleteSalesStats();
        Iterator it = ccs.keySet(null).iterator();
        //initialize m and y (don't init with -1 as this might wrongly cause directlyBefore() to be true if
        //the parameters are unfavorable
        int m = -2;
        int y = -2;
        //if there are already saved statistics, initialize m and y with the month directly before
        //the first statistics entry
        //this is necessary to prevent the iterator from iterating over ALL elements, if the first
        //statistics element to be evaluated is also the first one ever saved (the directlyBefore()
        //test would say true the first time, then m and y equal mFrom and yFrom or are greater
        //and the loop would only stop if the iterator reached the last element)
        if (it.hasNext()) {
            CSalesStats css = ccs.get((String)ccs.keySet(null).iterator().next()); //use new iterator!
            m = css.getMonth() - 1;
            y = css.getYear();
        }
        //iterate over statistics catalog, until the element directly before (mFrom, yFrom) is reached
        while (it.hasNext() && !directlyBefore(m, y, mFrom, yFrom)) {
            CSalesStats css = ccs.get((String)it.next());
            y = css.getYear();
            m = css.getMonth();
        }
        //iterate until (mTo, yTo) is reached, sum up all stats
        while (it.hasNext() && !(y == yTo && m == mTo)) {
            CSalesStats css = ccs.get((String)it.next());
            y = css.getYear();
            m = css.getMonth();
            CISalesStats ciss = css.get(id);
            returnStats.addAmount(ciss.getAmount());
            returnStats.addRevenue(ciss.getRevenue());
            returnStats.appendOrderHistory(ciss.getOrderHistory());
            returnStats.appendPriceHistory(ciss.getPriceHistory());
        }
        //if queried time range ends with current month, add current month's stats to returnStats,
        if (mTo == SMarket.getMonth() && yTo == SMarket.getYear()) {
            returnStats.addAmount(thisMonthsItem.getAmount());
            returnStats.addRevenue(thisMonthsItem.getRevenue());
            returnStats.appendOrderHistory(thisMonthsItem.getOrderHistory());
            returnStats.appendPriceHistory(thisMonthsItem.getPriceHistory());
        }
        return returnStats;
    }


    /**
     * @return {@link CCompleteStats CCompleteStats'} first entry.
     */
    public static CSalesStats getFirstArticleStatisticsEntry() {
        CCompleteStats ccs = SMarket.getCompleteSalesStats();
        Iterator it = ccs.keySet(null).iterator();
        CSalesStats css = null;
        if (it.hasNext()) {
            css = ccs.get((String)it.next());
        }
        return css;
    }

    /**
     * @return the year of CCompleteStats' first entry. If there is no first entry,
     * the current year is returned.
     */
    public static int getFirstArticleStatisticsYear() {
        CSalesStats css = getFirstArticleStatisticsEntry();
        return css == null ? SMarket.getYear() : css.getYear();
    }

    /**
     * @return the month of CCompleteStats' first entry. If there is no first entry,
     * the current month is returned.
     */
    public static int getFirstArticleStatisticsMonth() {
        CSalesStats css = getFirstArticleStatisticsEntry();
        return css == null ? SMarket.getMonth() : css.getMonth();
    }

    /**
     * @return CCompleteStats' last entry.
     */
    public static CSalesStats getLastArticleStatisticsEntry() {
        CCompleteStats ccs = SMarket.getCompleteSalesStats();
        Iterator it = ccs.keySet(null).iterator();
        CSalesStats css = null;
        while (it.hasNext()) {
            css = ccs.get((String)it.next());
        }
        return css;
    }


    /**
     * @return the year of CCompleteStats' last entry. If there is no entry,
     * the current year is returned.
     */
    public static int getLastArticleStatisticsYear() {
        CSalesStats css = getLastArticleStatisticsEntry();
        return css == null ? SMarket.getYear() : css.getYear();
    }

    /**
     * @return the month of CCompleteStats' last entry. If there is no entry,
     * the current month is returned.
     */
    public static int getLastArticleStatisticsMonth() {
        CSalesStats css = getLastArticleStatisticsEntry();
        return css == null ? SMarket.getMonth() : css.getMonth();
    }

    /**
     * Checks if a month is immediately before another one (e.g. March 2003 is directlyBefore April 2003).
     *
     * @param month the month that is checked
     * @param year the year in that is checked
     * @param limitMonth the month with which the checked month is compared.
     * @param limitYear the year with which the checked year is compared.
     *
     * @return <code>true</code> if the checked month is directly before the other one, otherwise
     * <code>false</code>.
     */
    private static boolean directlyBefore(int month, int year, int limitMonth, int limitYear) {
        return (month + 1 == limitMonth && year == limitYear) ||
                (month == 11 && limitMonth == 0 && year + 1 == limitYear);
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Customer stats
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * @param uc the customer whose statistics are of interest.
     * @return the summed value of all purchases the customer has made so far.
     */
    public static int getCompleteCustomerRevenue(UCustomer uc) {
        int sum = 0;
        Iterator it = SMarket.getCustomerStats().get(uc.getName()).getHistory().iterator();
        while (it.hasNext()) {
            sum += ((HistoryEntry)it.next()).getValue();
        }
        return sum;
    }

    /**
     * @param uc the customer whose allowable revenue is of interest.
     * @return the summed value of all purchases that affect the discount.
     */
    public static int getAllowableCustomerRevenue(UCustomer uc) {
        Calendar now = SMarket.getTime();
        //compute the first date, which is taken into account when calculating the discount
        Calendar firstDate = new MarketCalendar(now.get(Calendar.YEAR),
                now.get(Calendar.MONTH) - SMarket.getOptions().getDiscountRange(),
                now.get(Calendar.DATE));
        int sum = 0;
        Iterator it = SMarket.getCustomerStats().get(uc.getName()).getHistory().iterator();
        while (it.hasNext()) {
            HistoryEntry he = (HistoryEntry)it.next();
            if (!he.getDate().before(firstDate)) {
                sum += he.getValue();
            }
        }
        return sum;
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Overall stats
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Sums up the CSalesStats for the desired range of time.
     *
     * @param mFrom the first month of the statistics interval.
     * @param yFrom the first year of the statistics interval.
     * @param mTo the last month of the statistics interval.
     * @param yTo the last year of the statistics interval.
     *
     * @return the summed up CSalesStats.
     */
    public static CSalesStats getOverallStats(int mFrom, int yFrom, int mTo, int yTo) {
        CCompleteStats ccs = SMarket.getCompleteSalesStats();
        CSalesStats returnStats = new CSalesStats(0, 0);
        Iterator it = ccs.keySet(null).iterator();
        //initialize m and y (don't init with -1 as this might cause directlyBefore() to be true if
        //the parameters are unfavorable
        int m = -2;
        int y = -2;
        //if there are already saved statistics, initialize m and y with the month directly before
        //the first statistics entry
        //this is necessary to prevent the iterator from iterating over ALL elements. This would
        //happen if the first statistics element to be evaluated is also the first one ever saved.
        //(the directlyBefore() test would say true the first time, then m and y equal mFrom and yFrom
        //or are greater and the loop would only stop if the iterator reached the last element)
        if (it.hasNext()) {
            CSalesStats css = ccs.get((String)ccs.keySet(null).iterator().next()); //use new iterator!
            m = css.getMonth() - 1;
            y = css.getYear();
        }
        //iterate over statistics catalog, until the element directly before (mFrom, yFrom) is reached
        while (it.hasNext() && !directlyBefore(m, y, mFrom, yFrom)) {
            CSalesStats css = ccs.get((String)it.next());
            y = css.getYear();
            m = css.getMonth();
        }
        //iterate until (mTo, yTo) is reached, sum up all stats
        int marketPurchases = 0;
        while (it.hasNext() && !(y == yTo && m == mTo)) {
            CSalesStats css = ccs.get((String)it.next());
            y = css.getYear();
            m = css.getMonth();
            returnStats.setCosts(returnStats.getCosts() + css.getCosts());
            returnStats.setWages(returnStats.getWages() + css.getWages());
            returnStats.addRevenue(css.getRevenue());
            returnStats.addOrders(css);
        }
        //if queried time range ends with current month, add current month's stats to returnStats,
        if (mTo == SMarket.getMonth() && yTo == SMarket.getYear()) {
            CSalesStats now = SMarket.getMonthlySalesStats();
            returnStats.setCosts(returnStats.getCosts() + now.getCosts());
            returnStats.setWages(returnStats.getWages() + UMUserBase.getGlobalBase().getCurrentWages());
            returnStats.addRevenue(now.getRevenue());
            returnStats.addOrders(now);
        }
        return returnStats;
    }

}