package market.stdform;

import java.awt.Component;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;

import market.JDDShowMessage;
import market.swing.JTFCheckable;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import data.stdforms.SingleTableFormSheet;

/**
 * This special FormSheet wraps "normal" {@link FormSheet FormSheets} and provides the possibilty to both
 * check special {@link JTFCheckable checkable text fields} for valid entries and to retrieve those entries.
 * Furthermore, this class generates error messages in case one or more of those special text fields
 * contain invalid entries.
 * However, {@link JTFCheckable checkable text fields} have already to be defined in the original
 * FormSheet.
 */
public class FSCheckable extends FormSheet implements java.io.Serializable {

    /**
     * The original FormSheet to be wrapped.
     */
    private FormSheet formSheet;
    private List checkableComponents, invalidFields;
    public static final int NO_ERRORMESSAGE = 0;
    public static final int FIRST_ERRORMESSAGE = 1;
    public static final int ALL_ERRORMESSAGES_SEQUENTIALLY = 2;
    public static final int ALL_ERRORMESSAGES_AT_ONCE = 3;

    /**
     * Creates an empty FormSheet, sets the content and the buttons of the FormSheet
     * which is passed as parameter as its own.
     *
     * @param fs the FormSheet to be wrapped.
     */
    public FSCheckable(FormSheet fs) {
        super(fs.getCaption(), (JComponent)null);
        this.formSheet = fs;
        addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(FormSheet fsOrig) {
                fsOrig.setComponent(formSheet.getComponent());
                setButtons(getButtons());
            }
        });
        invalidFields = new ArrayList();
    }

    /**
     * @return a List of all buttons in the {@link #formSheet formSheet's} button bar
     */
    public List getButtons() {
        List buttonList = new LinkedList();
        Iterator it = formSheet.buttonIterator(false);
        while (it.hasNext()) {
            buttonList.add((FormButton)it.next());
        }
        return buttonList;
    }

    /**
     * Sets buttons to the button bar.
     *
     * @param buttonList the list of buttons to be set.
     */
    public void setButtons(List buttonList) {
        removeAllButtons();
        Iterator it = buttonList.iterator();
        while (it.hasNext()) {
            addButton((FormButton)it.next());
        }
    }


    /**
     * @param number the ID of the desired {@link JTFCheckable}
     * @return the text of a {@link JTFCheckable}
     */
    public String getEntry(int number) {
        Object listObject = null;
        JTFCheckable ctf = null;
        boolean found = false;
        checkableComponents = getCheckableComponents(getComponent());
        Iterator it = checkableComponents.iterator();
        while (it.hasNext() && !found) {
            ctf = (JTFCheckable)it.next();
            found = (ctf.getID() == number);
        }
        return found ? ctf.getText() : "";
    }

    /**
     * Checks all {@link JTFCheckable JTFCheckables} for validity entries.
     * @param option the style of the {@link #generateErrorMessage(int, boolean) error message} to be created.
     * @param sorted the order of the {@link #generateErrorMessage(int, boolean) error messages} to be created.
     * @return true, if all checkable text fields contain a valid value, else false.
     */
    public boolean checkTextFields(int option, boolean sorted) {
        boolean returnValue = true;
        boolean singleValue;
        JTFCheckable nextField = null;
        checkableComponents = getCheckableComponents(getComponent());
        Iterator it = checkableComponents.iterator();
        while (it.hasNext()) {
            nextField = (JTFCheckable)it.next();
            if (!nextField.hasValidValue()) {
                returnValue = false;
                invalidFields.add(nextField);
            };
        }
        if (!returnValue) {
            generateErrorMessage(option, sorted);
        }
        invalidFields.clear();
        return returnValue;
    }

    /**
     * Generates error messages for {@link JTFCheckable checkable text fields} that do not contain
     * valid values.
     * @param option defines style of the error message.
     *  <ul><li>0: show no error messages</li>
     *      <li>1: show only first error message</li>
     *      <li>2: show all error messages sequentially</li>
     *      <li>3: show all error messages at once</l>
     *  </ul>
     * @param sorted Select if the error messages should be displayed in the order the associated
     * {@link JTFCheckable checkable text fields} have been added to the FormSheet (false)<br>
     * or sorted by their ButtonIDs (true).
     */
    private void generateErrorMessage(int option, boolean sorted) {
        List invF = sorted ? sort(invalidFields) : invalidFields;
        Iterator it = invF.iterator();
        JTFCheckable ctf = null;
        switch (option) {
            case 1:
                JDDShowMessage.showMessageDialog(this, ((JTFCheckable)it.next()).getErrorMessage(),
                        "Fehlerhafte Eingabe");
                break;
            case 2:
                while (it.hasNext()) {
                    ctf = (JTFCheckable)it.next();
                    JDDShowMessage.showMessageDialog(this, ctf.getErrorMessage(), "Fehlerhafte Eingabe");
                }
               break;
            case 3:
                String s = "";
                while (it.hasNext()) {
                    ctf = (JTFCheckable)it.next();
                    s += "- " + ctf.getErrorMessage() + "\n";
                }
                JDDShowMessage.showMessageDialog(this, s, "Fehlerhafte Eingabe");
                break;
        }
    }

    /**
     * Sorts a List of {@link JTFCheckable checkable text fields} by their IDs.
     *
     * @param toSort the list of checkable text fields to be sorted.
     * @return the sorted list.
     */
    private List sort(List toSort) {
        Collections.sort(toSort, new Comparator() {
            public int compare(Object o1, Object o2) {
                JTFCheckable ctf1 = (JTFCheckable)o1;
                JTFCheckable ctf2 = (JTFCheckable)o2;
                return (ctf1.getID() - ctf2.getID());
            }
        });
        return toSort;
    }

    /**
     * Recursively searches a given JComponent for CheckableTextFields.
     *
     * @param comp the user interface's component to be searched upon.
     * @return a list of all {@link JTFCheckable checkable text fields}.
     */
    private List getCheckableComponents(JComponent comp) {
        List returnComponents = new ArrayList();
        Component[] comps = comp.getComponents();
        for (int i = 0; i < comps.length; i++) {
            //add all Components following recursions might find
            if (comps[i] instanceof JPanel) {
                returnComponents.addAll(getCheckableComponents((JComponent)comps[i]));
            };
            //check for Class market.stdform.JTFCheckable, if found add Component to list
            if (comps[i] instanceof JTFCheckable) {
                returnComponents.add(comps[i]);
            }
        }
        return returnComponents;
    }

    /**
     * @return the selected Record of an embedded {@link SingleTableFormSheet}.
     * If no SingleTableFormSheet is embedded, null is returned.
     */
    public Object getSelectedRecord() {
        if (formSheet instanceof SingleTableFormSheet) {
            return ((SingleTableFormSheet)formSheet).getSelectedRecord();
        } else {
            return null;
        }
    }

    /**
     * @return the FormSheet which is wrapped by this FSCheckable.
     */
    public FormSheet getFormSheet(){
        return this.formSheet;
    }
}