package market.stdform;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import market.Conversions;
import market.UCustomer;
import market.UPerson;
import market.UStaffer;
import market.VCDummy;
import market.VCPositiveDouble;
import market.VCPositiveInteger;
import market.swing.ComponentFactory;
import market.swing.JTFCheckable;
import sale.FormSheet;
import sale.FormSheetContentCreator;

/**
 * This FormSheet class is used in multiple cases, for both editing customers and employees.
 * The look of the FormSheet varies slightly depending on the type of person to be edited.
 * Customers have a field with their company and their discount, employees have a field with
 * their special occupation and their salary.
 * The types of usage for this FormSheet are:
 * <ul>
 * <li>For new customers to sign up: all fields but discount are editable.</li>
 * <li>For customers to edit their data: all fields but login and discount are enabled</li>
 * <li>For the manager to edit customers: only the password fields are enabled</li>
 * <li>For the manager to employ new workers: all fields are enabled.</li>
 * <li>For the manager to edit employees: all fields but login are enabled</li>
 * </ul>
 */
public class FSEditPersonData extends FormSheet{

    public static final int JTFC_FIRSTNAME = 0;
    public static final int JTFC_NAME = 1;
    public static final int JTFC_TELEPHONE = 2;
    public static final int JTFC_STREET = 3;
    public static final int JTFC_POSTCODE = 4;
    public static final int JTFC_CITY = 5;
    public static final int JTFC_LOGIN = 6;
    public static final int JTFC_COMPANY = 7;
    public static final int JTFC_SALARY = 8;
    public static final int JTFC_SECTION = 9;
    public static final int JTFC_DISCOUNT = 10;
    private static final int WIDTH = 15;
    private UPerson person;
    private JComboBox jcb_salutation;
    private JComboBox jcb_qualification;
    private JTFCheckable jtfc_firstName = new JTFCheckable(JTFC_FIRSTNAME, new VCDummy("Vorname"), WIDTH);
    private JTFCheckable jtfc_name = new JTFCheckable(JTFC_NAME, new VCDummy("Nachname"), WIDTH);
    private JTFCheckable jtfc_telephone = new JTFCheckable(JTFC_TELEPHONE,
            new VCDummy("Telefon", true), WIDTH);
    private JTFCheckable jtfc_street = new JTFCheckable(JTFC_STREET, new VCDummy("Strae"), WIDTH);
    private JTFCheckable jtfc_postcode = new JTFCheckable(JTFC_POSTCODE,
            new VCPositiveInteger("Postleitzahl"), WIDTH);
    private JTFCheckable jtfc_city = new JTFCheckable(JTFC_CITY, new VCDummy("Stadt"), WIDTH);
    private JTFCheckable jtfc_login = new JTFCheckable(JTFC_LOGIN, new VCDummy("Login"), WIDTH);
    private JTFCheckable jtfc_company = new JTFCheckable(JTFC_COMPANY, new VCDummy("Firma"), 10);
    private JTFCheckable jtfc_discount = new JTFCheckable(JTFC_DISCOUNT, new VCDummy("Rabatt"), 10);
    private JTFCheckable jtfc_salary = new JTFCheckable(JTFC_SALARY, new VCPositiveDouble("Gehalt"), WIDTH);
    private JPasswordField jpf_password = new JPasswordField(WIDTH);
    private JPasswordField jpf_confirm = new JPasswordField(WIDTH);
    private JTextField jtf_Date = new JTextField(WIDTH);

    /**
     * Creates a new FSEditPersonData FormSheet
     *
     * @param up the user to be edited, if <code>null</code> the fields stay empty.
     * @param option an option to specify the special look of the FormSheet.
     * <ul>
     * <li>0 customer edits his profile</li>
     * <li>1 manager edits customer profile</li>
     * <li>2 manager edits worker profile</li>
     * </ul>
     */
    public FSEditPersonData(UPerson up, final int option){
        super("Personendaten", null);
        this.person = up;
        this.addContentCreator(new FormSheetContentCreator(){
            protected void createFormSheetContent(FormSheet fs) {
                fs.removeAllButtons();
                JPanel jp_m = new JPanel();
                JPanel jp_main = new JPanel();
                JPanel jp_person = new JPanel();
                JPanel jp_address = new JPanel();
                JPanel jp_login = new JPanel();
                jcb_salutation = new JComboBox(new Object[] {UPerson.FRAU, UPerson.HERR});
                jcb_qualification = new JComboBox(new Object[]
                        {UStaffer.SELLER, UStaffer.WAREHOUSE_WORKER, UStaffer.MANAGER});
                jcb_qualification.setPreferredSize(new Dimension(15,10));
                jcb_salutation.setPreferredSize(new Dimension(15,10));
                GridBagLayout gridbag = new GridBagLayout();
                GridBagConstraints c = new GridBagConstraints();
                jp_m.setLayout(gridbag);
                    c.weighty = 1;
                    c.anchor = GridBagConstraints.CENTER;
                gridbag.setConstraints(jp_main, c);
                jp_m.add(jp_main);
                jp_main.setLayout(new BoxLayout(jp_main, BoxLayout.Y_AXIS));
                jp_person.setLayout(new GridLayout(4, 2));
                jp_person.setBorder(ComponentFactory.createInsetBorder("Personendaten"));
                jp_address.setLayout(new GridLayout(3, 2));
                jp_address.setBorder(ComponentFactory.createInsetBorder("Adressdaten"));
                jp_login.setLayout(new GridLayout(3, 2));
                jp_login.setBorder(ComponentFactory.createInsetBorder("Login"));
                jp_person.add(new JLabel("Anrede:"));                jp_person.add(jcb_salutation);
                jp_person.add(new JLabel("Vorname:"));               jp_person.add(jtfc_firstName);
                jp_person.add(new JLabel("Nachname:"));              jp_person.add(jtfc_name);
                jp_person.add(new JLabel("Telefon:"));               jp_person.add(jtfc_telephone);
                jp_address.add(new JLabel("Strae:"));               jp_address.add(jtfc_street);
                jp_address.add(new JLabel("Postleitzahl:"));         jp_address.add(jtfc_postcode);
                jp_address.add(new JLabel("Stadt:"));                jp_address.add(jtfc_city);
                jp_login.add(new JLabel("Login:"));                  jp_login.add(jtfc_login);
                jp_login.add(new JLabel("Passwort:"));               jp_login.add(jpf_password);
                jp_login.add(new JLabel("Passwort-Wiederholung:"));  jp_login.add(jpf_confirm);
                jtfc_discount.setEditable(false);
                jp_main.add(jp_person);
                jp_main.add(jp_address);
                jp_main.add(jp_login);
                if (option == 0 || option == 1) {
                    jp_main.add(getCompanyPanel());
                }
                if (option == 2) {
                    jp_main.add(getQualificationPanel());
                }
                fs.addButton("OK", ButtonIDs.BTN_ACCEPT, null);
                fs.addButton("Zurck", ButtonIDs.BTN_BACK, null);
                fs.setComponent(jp_m);
            }
        });
        // if a person was referred, the textfields are filled with the person's data
        if (person != null) {
            jcb_salutation.setSelectedItem(person.getSalutation());
            jtfc_firstName.setText(person.getFirstName());
            jtfc_name.setText(person.getSurname());
            jtfc_telephone.setText(person.getTelephone());
            jtfc_street.setText(person.getStreet());
            jtfc_postcode.setText(String.valueOf(person.getPostcode()));
            jtfc_city.setText(person.getCity());
            jtfc_login.setText(person.getName());
            jtfc_login.setEditable(false);      //login can never be changed
            if (person instanceof UCustomer) {
                UCustomer p = (UCustomer)person;
                jtfc_company.setText(p.getCompany());
                jtfc_discount.setText(Conversions.fixedDecimal(100 * p.getDiscount(), 3) + " %");
            }
            if (person instanceof UStaffer) {
                UStaffer p = (UStaffer)person;
                jcb_qualification.setSelectedItem(p.getQualification());
                jtfc_salary.setText(Conversions.valueToCurrency(p.getSalary()));
            }
            jtf_Date.setText(person.getDayOfRegistration().toString());
        }
        if (option == 1) {              //if manager view customer's data...
            jcb_salutation.setEnabled(false);           //disable all but password
            jtfc_name.setEditable(false);
            jtfc_firstName.setEditable(false);
            jtfc_telephone.setEditable(false);
            jtfc_street.setEditable(false);
            jtfc_postcode.setEditable(false);
            jtfc_city.setEditable(false);
            jtfc_login.setEditable(false);
            jtfc_company.setEditable(false);
        }
    }

    /**
     * @return the panel with textfields which are used for customers only.
     */
    private JPanel getCompanyPanel(){
        JPanel jp_company = new JPanel();
        jp_company.setLayout(new GridLayout(3,2));
        jp_company.setBorder(ComponentFactory.createInsetBorder("Sonstiges"));
        jp_company.add(new JLabel("Firma: "));
        jp_company.add(jtfc_company);
        jp_company.add(new JLabel("Rabatt: "));
        jtfc_discount.setText(Conversions.fixedDecimal(0, 3));
        jp_company.add(jtfc_discount);
        jp_company.add(new JLabel("Mitglied seit: "));
        jp_company.add(jtf_Date);
        jtf_Date.setEditable(false);
        return jp_company;
    }

    /**
     * @return the panel with textfields which are used for employees only.
     */
    private JPanel getQualificationPanel(){
        JPanel jp_qualification = new JPanel();
        jp_qualification.setLayout(new GridLayout(3,2));
        jp_qualification.setBorder(ComponentFactory.createInsetBorder("Beschftigung"));
        jp_qualification.add(new JLabel("Beschftigung: "));
        jp_qualification.add(jcb_qualification);
        jp_qualification.add(new JLabel("Gehalt: "));
        jp_qualification.add(jtfc_salary);
        jp_qualification.add(new JLabel("Angestellt seit: "));
        jp_qualification.add(jtf_Date);
        jtf_Date.setEditable(false);
        return jp_qualification;
    }

    /**
     * Checks if password and confirmation match.
     * @return <code>true</code> if password and confirmation match, otherwise <code>false</code>.
     */
    public boolean passwordsEqual(){
        String s1 = String.valueOf(jpf_password.getPassword());
        String s2 = String.valueOf(jpf_confirm.getPassword());
        return s1.compareTo(s2) == 0;
    }

    /**
     * @return the password typed into the password field.
     */
    public char[] getPassword(){
        if(passwordsEqual() && (jpf_password.getPassword().length > 0)) {
            return jpf_password.getPassword();
        }
        else return null;
    }

    /**
     * Checks if a password has been set.
     * @return <code>true</code> if a password has been set, otherwise <code>false</code>.
     */
    public boolean isPasswordSet() {
        return jpf_password.getPassword().length > 0 && jpf_confirm.getPassword().length > 0;
    }

    /**
     * @return value of the salutation ComboBox.
     */
    public String getSalutation(){
        return (String)jcb_salutation.getSelectedItem();
    }

    /**
     * @return the value of the occupation ComboBox.
     */
    public String getQualification(){
        return (String)jcb_qualification.getSelectedItem();
    }

    /**
     * Creation method for this FormSheet if it is to be used by customers.
     * @param customer the customer to be displayed with the FormSheet.
     * @return the FormSheet as {@link FSCheckable}.
     */
    public static FSCheckable getCustomerProfileForCustomer(UCustomer customer) {
        FormSheet fs = new FSEditPersonData(customer, 0);
        return new FSCheckable(fs);
    }

    /**
     * Creation method for this FormSheet if it is to be used by the manger to view customers' data.
     * @param customer the customer to be displayed with the FormSheet.
     * @return the FormSheet as {@link FSCheckable}.
     */
    public static FSCheckable getCustomerProfileForManager(UCustomer customer){
        FormSheet fs = new FSEditPersonData(customer, 1);
        return new FSCheckable(fs);
    }

    /**
     * Creation method for this FormSheet if it is to be used by the manger to view employees' data.
     * @param staffer the employee to be displayed with the FormSheet.
     * @return the FormSheet as {@link FSCheckable}.
     */
    public static FSCheckable getStafferProfile(UStaffer staffer){
        FormSheet fs = new FSEditPersonData(staffer, 2);
        return new FSCheckable(fs);
    }
}