package market.stdform;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Comparator;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import market.CIArticle;
import market.Conversions;
import market.SMarket;
import market.swing.CCSStrategyMarket;
import market.swing.CmpNumbers;
import market.swing.JTADescriptionArea;
import sale.FormSheet;
import sale.FormSheetContentCreator;
import sale.UIGate;
import util.swing.AbstractTableEntryDescriptor;
import data.CountingStock;
import data.DataBasket;
import data.stdforms.TwoTableFormSheet;
import data.swing.CountingStockTableModel;


/**
 * This FormSheet is used by the manager to purchase new items for the market's stock.
 */
public class FSManagerPurchase {

    /**
     * Creates a {@link TwoTableFormSheet}. The look of the table is defined by the
     * {@link TEDManagerPurchase}.
     *
     * @param gate the gate on which the TwoTableFormSheet should be displayed. It must not be null,
     * because a valid gate is needed to enable the shifting of items between the tables.
     * @param cs the CountingStock in which the items that the manager selected for purchase are stored.
     * @param db the transaction's DataBasket.
     *
     * @return the created TwoTableFormSheet.
     */
    public static TwoTableFormSheet create(UIGate gate, CountingStock cs, DataBasket db) {
        final TwoTableFormSheet ttfs = TwoTableFormSheet.create(
            "Einkauf",                                                    //Caption
            SMarket.getArticleCatalog(),                                  //TableSources
            cs,
            db,                                                           //DB
            gate,                                                         //UIGate
            null, null,                                                   //Initial Comparators
            false,                                                        //Display zeros
            new TEDManagerPurchase(true), new TEDManagerPurchase(false),  //TableEntryDescriptors
            new CCSStrategyMarket()                                       //MoveStrategy
        );
        ttfs.addContentCreator(new FormSheetContentCreator() {
            protected void createFormSheetContent(final FormSheet fs) {
                //define components
                JPanel jpMain = new JPanel();
                JPanel jpUpper = new JPanel();
                JPanel jpTables = new JPanel();
                JPanel jpDescription = new JPanel();
                JScrollPane jsc = new JScrollPane();
                final JTADescriptionArea da = new JTADescriptionArea();
                GridBagConstraints c = new GridBagConstraints();
                GridBagLayout gridbag = new GridBagLayout();
                //add Listener to table that reacts on selection change
                ttfs.getLeftTable().getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                    public void valueChanged(ListSelectionEvent e) {
                        if (!e.getValueIsAdjusting()) {
                            da.setDescription(Conversions.recordToCIArticle(
                                ttfs.getLeftSelectedRecord()));
                        }
                    }
                });
                //add components
                jpTables = (JPanel)fs.getComponent();
                jpDescription.setLayout(new BoxLayout(jpDescription, BoxLayout.X_AXIS));
                jpDescription.add(jsc);
                    jsc.setViewportView(da);
                jpMain.setLayout(new BoxLayout(jpMain, BoxLayout.Y_AXIS));
                jpMain.add(jpTables);
                jpMain.add(Box.createVerticalStrut(10));
                jpMain.add(jpDescription);
                fs.setComponent(jpMain);
                fs.removeAllButtons();
                fs.addButton("Kaufen", ButtonIDs.BTN_BUY, null);
            }
        });
        return ttfs;
    }
}

/**
 * The {@link util.swing.TableEntryDescriptor} used by {@link FSManagerPurchase}.
 */
class TEDManagerPurchase extends AbstractTableEntryDescriptor {

    /**
     * Is queried to determine whether this TED is used for the left or for the right table.
     */
    private boolean left;
    private Comparator sortOfferSum = new CmpNumbers(CmpNumbers.OFFER, CmpNumbers.SUMMED);
    private Comparator sortOfferNoSum = new CmpNumbers(CmpNumbers.OFFER, CmpNumbers.SINGLE);
    private Comparator sortCount = new CmpNumbers(CmpNumbers.COUNT);

    /**
     * @param left <ul><li>true: Use this TableEntryDescriptor for the left table.</li>
     *                  <li>false: Use this TableEntryDescriptor for the right table.</li</ul>
     */
    public TEDManagerPurchase(boolean left) {
        this.left = left;
    }

    /**
     * @return the number of the table's columns.
     */
    public int getColumnCount() {
        return 3;
    }

    /**
     * @param nIndex the affected column.
     * @return columns' names.
     */
    public String getColumnName(int nIndex) {
        return (new String[]{ "Artikel", left ? "Kategorie" : "Preis", left ? "Preis" : "Anzahl"}) [nIndex];
    }

    /**
     * @param nIndex the affected column.
     * @return columns' classes. They indicate how column's values should be aligned.
     */
    public Class getColumnClass  (int nIndex) {
        return (new Class[] {String.class, left ? String.class : Number.class, Number.class}) [nIndex];
    }

    /**
     * @param oRecord the affected table record.
     * @param nIndex the affected column.
     * @return columns' values
     */
    public Object getValueAt(Object oRecord, int nIndex) {
        CountingStockTableModel.Record stockitem = null;
        CIArticle article = Conversions.recordToCIArticle(oRecord);
        int count = 0;
        if (!left) {
            count = ((CountingStockTableModel.Record)oRecord).getCount();
        }
        switch (nIndex) {
            case 0: return article.getArticleName();
            case 1: return left ? article.getCategory() :
                            Conversions.doubleToCurrency(count * article.getOffer());
            case 2: return left ? Conversions.doubleToCurrency(article.getOffer()) :
                            new Integer(count).toString();
        }
        return null;
    }

    /**
     * Determines if columns can be sorted by the user.
     *
     * @param nIndex the affected column.
     * @return <ul><li>true: columns can be sorted</li>
     *              <li>false: columns cannot be sorted</li></ul>
     */
    public boolean canSortByColumn(int nIndex) {
        return true;
    }

    /**
     * @param nIndex the affected column.
     * @return the {@link Comparator} to be used when sorting the column.
     */
    public Comparator getColumnOrder(int nIndex) {
        switch(nIndex) {
            case 1: return left ? null : sortOfferSum;
            case 2: return left ? sortOfferNoSum : sortCount;
        }
        return null;
    }
}