package market.swing;

import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JTextField;
import javax.swing.border.Border;

/**
 * Factory class for borders, JTextFields und {@link JTFCheckable JTFCheckables}.
 */
public class ComponentFactory {

    public static final int LEFT = JTextField.LEFT;
    public static final int CENTER = JTextField.CENTER;
    public static final int RIGHT = JTextField.RIGHT;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// borders
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a line border out that leaves some space around the object to which the border is set.
     * This is achieved with the help of an empty border between the object and the line border.
     *
     * @return the border.
     */
    public static Border createInsetBorder() {
        return createInsetBorder(BorderFactory.createLineBorder(Color.gray), 5, 10, 10, 10);
    }

    /**
     * Creates a title border out that leaves some space around the object to which the border is set.
     * This is achieved with the help of an empty border between the object and the title border.
     *
     * @param label the caption of the title border.
     * @return the border.
     */
    public static Border createInsetBorder(String label) {
        return createInsetBorder(BorderFactory.createTitledBorder(label), 5, 10, 10, 10);
    }

    /**
     * Creates a compound border out that leaves some space around the object to which the border is set.
     * This is achieved with the help of an empty border between the object and the title border.
     *
     * @param border the outer border.
     * @param top the top inset.
     * @param left the left inset.
     * @param bottom the bottom inset.
     * @param right the right inset.
     * @return the border.
     */
    public static Border createInsetBorder(Border border, int top, int left, int bottom, int right) {
        return BorderFactory.createCompoundBorder(border, BorderFactory.createEmptyBorder(top, left, bottom,
                right));
    }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// JTextFields
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Creates a JTextField.
     * @param text the text to be set.
     * @param size the text field's size
     * @param editable defines if text field is editable or not
     * @param alignment the text's alignment in the text field (left, center, right)
     * @param border defines if the text field should have a border.
     * @return the created JTextField.
     */
    public static JTextField createTextField(String text, int size, boolean editable, int alignment,
            boolean border) {
        return createTextField(text, size, editable, alignment, Color.black, border);
    }

    /**
     * Creates a JTextField.
     * @param value the number to be set as text.
     * @param size the text field's size
     * @param editable defines if text field is editable or not
     * @param alignment the text's alignment in the text field (left, center, right)
     * @param border defines if the text field should have a border.
     * @return the created JTextField.
     */
    public static JTextField createTextField(double value, int size, boolean editable, int alignment,
            boolean border) {
        return createTextField(Double.toString(value), size, editable, alignment, Color.black, border);
    }

    /**
     * Creates a JTextField.
     * @param text the text to be set.
     * @param size the text field's size
     * @param editable defines if text field is editable or not
     * @param alignment the text's alignment in the text field (left, center, right)
     * @param color the color of the text field's text.
     * @param border defines if the text field should have a border.
     * @return the created JTextField.
     */
    public static JTextField createTextField(String text, int size, boolean editable, int alignment,
            Color color, boolean border) {
        JTextField jtf = new JTextField(text, size);
        jtf.setEditable(editable);
        jtf.setHorizontalAlignment(alignment);
        jtf.setForeground(color);
        if (!border) {
            jtf.setBorder(null);
        }
        return jtf;
    }
}