package market.swing;

import java.awt.Dimension;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.Serializable;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;

import market.SMarket;
import market.event.MarketEventAdapter;
import market.statistics.Statistics;

/**
 * A set of {@link JComboBox JComboBoxes} for specifying a range of time for which statistics should
 * be displayed.
 */
public class JCTimeRangeBoxes implements Serializable {

    /**
     * The JComboBox for the start date's month.
     */
    private JComboBox jcbFromMonth = new JComboBox();

    /**
     * The JComboBox for the start date's year.
     */
    private JComboBox jcbFromYear = new JComboBox();

    /**
     * The JComboBox for the finish date's month.
     */
    private JComboBox jcbToMonth = new JComboBox();

    /**
     * The JComboBox for the finish date's year.
     */
    private JComboBox jcbToYear = new JComboBox();
    private Object[] oFromMonth, oFromYear, oToMonth, oToYear;
    private Dimension monthBoxDimension = new Dimension(100,25);
    private Dimension yearBoxDimension = new Dimension(65,25);

    public JCTimeRangeBoxes() {
        //both preferredSize and minimumSize have to be set:
        //preferredSize for the size the JComboBox is actually displayed with its borders
        //minimumSize for the size the BoxLayout manager reserves for displaying the JComboBox
        jcbFromYear.setPreferredSize(yearBoxDimension);
        jcbFromYear.setMinimumSize(yearBoxDimension);
        jcbToYear.setPreferredSize(yearBoxDimension);
        jcbToYear.setMinimumSize(yearBoxDimension);
        jcbFromMonth.setPreferredSize(monthBoxDimension);
        jcbFromMonth.setMinimumSize(monthBoxDimension);
        jcbToMonth.setPreferredSize(monthBoxDimension);
        jcbToMonth.setMinimumSize(monthBoxDimension);
        init();
        SMarket.addEventListener(new MarketEventAdapter() {
            public void timeAdvanced() {
                init();
            }
        });
        jcbFromYear.addItemListener(new FromItemListener());
        jcbToYear.addItemListener(new ToItemListener());
    }

    /**
     * Initializes the ComboBoxes' models according to the range of time from which
     * statistics can be displayed.
     */
    public void init() {
        jcbFromYear.setModel(new DefaultComboBoxModel(Statistics.createArticleStatisticsYears()));
        jcbToYear.setModel(new DefaultComboBoxModel(Statistics.createArticleStatisticsYears()));
        jcbFromMonth.setModel(new DefaultComboBoxModel(Statistics.createArticleStatisticsMonths(
                jcbFromYear.getSelectedItem())));
        jcbToMonth.setModel(new DefaultComboBoxModel(Statistics.createArticleStatisticsMonths(
                jcbFromYear.getSelectedItem())));
    }

    /**
     * @return the index of the month displayed by {@link #jcbFromMonth}.
     */
    public int getFromMonth() {
        return getMonth((String)jcbFromMonth.getSelectedItem());
    }

    /**
     * @return the index of the month displayed by {@link #jcbToMonth}.
     */
    public int getToMonth() {
        return getMonth((String)jcbToMonth.getSelectedItem());
    }

    /**
     * @return the index of the month displayed by {@link #jcbFromYear}.
     */
    public int getFromYear() {
        return new Integer((String)jcbFromYear.getSelectedItem()).intValue();
    }

    /**
     * @return the index of the month displayed by {@link #jcbToYear}.
     */
    public int getToYear() {
        return new Integer((String)jcbToYear.getSelectedItem()).intValue();
    }

    /**
     * @param s the name of a month (e.g. Januar or Mrz).
     * @return the index of the searched month according to the
     * {@linkplain market.statistics.Statistics#MONTHS months array}.
     */
    public int getMonth(String s) {
        Object[] months = Statistics.MONTHS;
        for (int i = 0; i <= 11; i++) {
            if (months[i].equals(s)) return i;
        }
        return -1;
    }

    /**
     * Checks if the start of the desired range of time is not after the end of the time range.
     * @return <code>true</code> if a valid time range was selected, otherwise <code>false</code>.
     */
    public boolean isValidTimeRange() {
        return (getFromYear() < getToYear()) ||
                (getFromYear() == getToYear() && getFromMonth() <= getToMonth());
    }

    /**
     * Checks if two {@link ComboBoxModel ComboBoxModels} are equal.<br>
     * This method is used to check if the available months of a box changed. This can happen when
     * the year's box is changed.
     *
     * @see FromItemListener
     * @see ToItemListener
     * @param cbm1 first ComboBoxModel to be compared.
     * @param cbm2 second ComboBoxModel to be compared.
     * @return <code>true</code> if both models are equal, otherwise <code>false</code>.
     */
    private boolean areModelsEqual(ComboBoxModel cbm1, ComboBoxModel cbm2) {
        String firstOld = (String)cbm1.getElementAt(0);
        String lastOld = (String)cbm1.getElementAt(cbm1.getSize() - 1);
        String firstNew = (String)cbm2.getElementAt(0);
        String lastNew = (String)cbm2.getElementAt(cbm2.getSize() - 1);
        return firstOld.equals(firstNew) && lastOld.equals(lastNew);
    }

    /**
     * Checks if a month is contained in a JComboBox.<br>
     * @see FromItemListener
     * @see ToItemListener
     * @param jcb the JComboBox to be examined.
     * @param month the searched month.
     * @return <code>true</code> if the month is part of the JComboBox, otherwise <code>false</code>.
     */
    private boolean containsMonth(JComboBox jcb, int month) {
        //searched month later than last month in box?
        if (month > jcb.getItemCount() - 1) return false;
        //searched month earlier than first month in box?
        int firstMonthOfBox = getMonth((String)jcb.getItemAt(0));
        return !(firstMonthOfBox > month);
    }

    /**
     * @return {@link #jcbFromMonth}.
     */
    public JComboBox getFromMonthBox() {
        return jcbFromMonth;
    }

    /**
     * @return {@link #jcbFromYear}.
     */
    public JComboBox getFromYearBox() {
        return jcbFromYear;
    }

    /**
     * @return {@link #jcbToMonth}.
     */
    public JComboBox getToMonthBox() {
        return jcbToMonth;
    }

    /**
     * @return {@link #jcbToYear}.
     */
    public JComboBox getToYearBox() {
        return jcbToYear;
    }

   /**
    * Listens to state changes on the {@link #jcbFromYear}.<br><br>
    * Whenever a new year is selected, the {@linkplain #jcbFromMonth month's values} will be
    * {@linkplain Statistics#createArticleStatisticsMonths recomputed}.<br>
    * If the JComboBoxes' models {@linkplain #areModelsEqual(ComboBoxModel, ComboBoxModel) are not equal},
    * i.e. months were added or removed, the new model is set. But that causes the JComboBox to be set
    * to its first value, even if that's not necessary.<br>
    * That's why there is another check, namely if the originally displayed month
    * {@linkplain #containsMonth(JComboBox, int) is contained} in the new JComboBox. If so, the
    * JComboBox will be set to this month.
    */
    private class FromItemListener implements ItemListener, Serializable {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == e.SELECTED) {
                ComboBoxModel cbm = new DefaultComboBoxModel(
                        Statistics.createArticleStatisticsMonths(
                        jcbFromYear.getSelectedItem()));               //update list of available months
                if (!areModelsEqual(jcbFromMonth.getModel(), cbm)) {   //if number of available months changed
                    int oldMonthSet = getFromMonth();                  //save currently displayed month
                    String oldMonthString = (String)jcbFromMonth.getSelectedItem();
                    jcbFromMonth.setModel(cbm);                        //set new model
                    if (containsMonth(jcbFromMonth, oldMonthSet)) {    //if saved month contained in new model
                        jcbFromMonth.setSelectedItem(oldMonthString);  //set box to that month
                    }
                }
            }
        }
    }

    /**
     * Listens to state changes on the {@link #jcbToYear}.<br><br>
     * Whenever a new year is selected, the {@linkplain #jcbToMonth month's values} will be
     * {@linkplain Statistics#createArticleStatisticsMonths recomputed}.<br>
     * If the JComboBoxes' models {@linkplain #areModelsEqual(ComboBoxModel, ComboBoxModel) are not equal},
     * i.e. months were added or removed, the new model is set. But that causes the JComboBox to be set
     * to its first value, even if that's not necessary.<br>
     * That's why there is another check, namely if the originally displayed month
     * {@linkplain #containsMonth(JComboBox, int) is contained} in the new JComboBox. If so, the
     * JComboBox will be set to this month.
     */
    private class ToItemListener implements ItemListener, Serializable {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == e.SELECTED) {
                ComboBoxModel cbm = new DefaultComboBoxModel(
                        Statistics.createArticleStatisticsMonths(
                        jcbToYear.getSelectedItem()));                //update list of available months
                if (!areModelsEqual(jcbToMonth.getModel(), cbm)) {    //if number of available months changed
                    int oldMonthSet = getToMonth();                   //save currently displayed month
                    String oldMonthString = (String)jcbToMonth.getSelectedItem();
                    jcbToMonth.setModel(cbm);                         //set new model
                    if (containsMonth(jcbToMonth, oldMonthSet)) {     //if saved month contained in new model
                        jcbToMonth.setSelectedItem(oldMonthString);   //set box to that month
                    }
                }
            }
        }
    }

}