package market.swing;

import javax.swing.JTextField;

import market.ValueChecker;

/**
 * A text field, which is provided with an ID that makes it possible to query its contents from outside
 * the defining class, without the need to write a get-method or even have the text field declared as
 * public.<br>
 * Furthermore, a {@link ValueChecker} is assigned, so that the text field's content can be easily checked
 * for validity. This results in a huge reduction of redundant code that would be necessary when checking
 * every text field manually.
 * A JTFCheckable, however, works only in connection with a {@link market.stdform.FSCheckable}.
 */
public class JTFCheckable extends JTextField {

    private ValueChecker chk;
    private int id;

    /**
     * @param id the ID of the JTFCheckable. Every ID must be unique to make sure the {@link market.stdform.FSCheckable}
     * works properly.
     * @param chk the ValueChecker for this JTFCheckable. It must not be <code>null</code>. If no checking
     * is desired, use {@link market.VCDummy} instead.
     * @param size the size of the JTFCheckable.
     */
    public JTFCheckable(int id, ValueChecker chk, int size) {
        super(size);
        this.id = id;
        this.chk = chk;
    }

    /**
     * @param id the ID of the JTFCheckable. Every ID must be unique to make sure the {@link market.stdform.FSCheckable}
     * works properly.
     * @param chk the ValueChecker for this JTFCheckable.
     * @param str the content of the JTFCheckable right after creation.
     * @param size the size of the JTFCheckable.
     */
    public JTFCheckable(int id, ValueChecker chk, String str, int size) {
        super(str, size);
        this.id = id;
        this.chk = chk;
    }

    /**
     * Checks the text field's content for validity with the help of the assigned {@link ValueChecker}.
     *
     * @return <code>true</code> if the content is valid, otherwise <code>false</code>.
     */
    public boolean hasValidValue() {
        return chk.isValidValue(this.getText());
    }

    /**
     * Asks the {@link ValueChecker} for an error message according to the kind of error.
     * @return the picked error message.
     */
    public String getErrorMessage() {
        return chk.getErrorString();
    }

    /**
     * @return the ID.
     */
    public int getID() {
        return id;
    }
}